/******************************************************************************/
/*   Copyright (C) 2001 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "n2type.h"
#include "n2ext.h"
/******************************************************************************/
/*                 Get the transmitter and receiver trellis.
*******************************************************************************/
int Get_Trellis()
{
 Get_Tx_Trellis();
 Get_Rx_Trellis();
// Modify_Rx_Trellis();
}
/******************************************************************************/
/*                      Get the transmitter trellis.
*******************************************************************************/
int Get_Tx_Trellis()
{
 int p_state,p_input;
 for(p_state=ZERO;p_state<NUM_TX_STATES;p_state++)
 {
  for(p_input=ZERO;p_input<NUM_INPUTS;p_input++)
  {
   Tx_Trel_Op[p_state][p_input]=Get_Tx_Trel_Op(p_state,p_input);
   Tx_Trel_NS[p_state][p_input]=(2*p_state+p_input)%NUM_TX_STATES;
   printf("\t Tx. trellis Pre. state:%3d Input:%3d ",p_state,p_input);
   printf("Nxt. state:%3d ",Tx_Trel_NS[p_state][p_input]);
   printf("Op:%3d.\n",Tx_Trel_Op[p_state][p_input]);
  }
 }
 printf("\n");
 return(0);
}
/******************************************************************************/
/*                   Get the output. LSB maps to real part.
*******************************************************************************/
int Get_Tx_Trel_Op(p_state,p_input)
int p_state,p_input;
{
 int bit0,bit1,op0,op1,dec_op;
 bit0=p_state&1;
 bit1=(p_state/2)&1;
 op0=p_input^bit1;
 op1=p_input^bit0^bit1;
 dec_op=op0+2*op1;
 return(dec_op);
}
/******************************************************************************/
/*                      Get the trellis at the receiver.
*******************************************************************************/
int Get_Rx_Trellis()
{
 int p_state,p_input;
 for(p_state=ZERO;p_state<NUM_RX_STATES;p_state++)
 {
  for(p_input=ZERO;p_input<NUM_INPUTS;p_input++)
  {
   Process_Rx_Trel(p_state,p_input);
   Print_Rx_Trellis(p_state,p_input);
  }
 }
 return(0);
}
/******************************************************************************/
/*          Process the receiver trellis for a given state and input.
*******************************************************************************/
int Process_Rx_Trel(p_state,p_input)
int p_state,p_input;
{
 int enc_state,enc_op,enc_ns,chan_state,chan_st_array[CHAN_MEM_SIZE];
 enc_state=p_state/NUM_CHAN_STATES;
 enc_op=Tx_Trel_Op[enc_state][p_input];
 enc_ns=Tx_Trel_NS[enc_state][p_input];
 chan_state=p_state%NUM_CHAN_STATES;
 Decimal_To_M_ary(chan_state,chan_st_array,CHAN_MEM_SIZE);
 Get_Rx_Trel_Op(p_state,p_input,enc_op,chan_st_array);
 Get_Rx_Trel_NS(p_state,p_input,enc_op,enc_ns,chan_st_array);
 return(0);
}
/******************************************************************************/
/* Get the outputs for the receiver trellis, given the input and the
   current state.
*******************************************************************************/
int Get_Rx_Trel_Op(p_state,p_input,enc_op,chan_st_array)
int p_state,p_input,enc_op,chan_st_array[CHAN_MEM_SIZE];
{
 int cnt;
 double re_sym,im_sym,re_tap,im_tap,re_op,im_op;
 re_sym=Re_Map[enc_op];
 im_sym=Im_Map[enc_op];
 re_tap=Re_Chan_Tap[ZERO];
 im_tap=Im_Chan_Tap[ZERO];
 re_op=re_tap*re_sym-im_tap*im_sym;
 im_op=re_tap*im_sym+im_tap*re_sym;
 for(cnt=ZERO;cnt<CHAN_MEM_SIZE;cnt++)
 {
  enc_op=chan_st_array[cnt];
  re_sym=Re_Map[enc_op];
  im_sym=Im_Map[enc_op];
  re_tap=Re_Chan_Tap[cnt+ONE];
  im_tap=Im_Chan_Tap[cnt+ONE];
  re_op=re_op+re_tap*re_sym-im_tap*im_sym;
  im_op=im_op+re_tap*im_sym+im_tap*re_sym;
 }
 Re_Rx_Trel_Op[p_state][p_input]=re_op;
 Im_Rx_Trel_Op[p_state][p_input]=im_op;
 return(0);
}
/******************************************************************************/
/* Get the next state for the receiver trellis, given the input and the
   current state.
*******************************************************************************/
int Get_Rx_Trel_NS(p_state,p_input,enc_op,enc_ns,chan_st_array)
int p_state,p_input,enc_op,enc_ns,chan_st_array[CHAN_MEM_SIZE];
{
 int cnt,chan_state;
 for(cnt=CHAN_MEM_SIZE-ONE;cnt>ZERO;cnt--)
  chan_st_array[cnt]=chan_st_array[cnt-ONE];
 chan_st_array[ZERO]=enc_op;
 chan_state=M_ary_To_Decimal(chan_st_array,CHAN_MEM_SIZE);
 Rx_Trel_NS[p_state][p_input]=enc_ns*NUM_CHAN_STATES+chan_state;
 return(0);
}
/******************************************************************************/
/*                        Print the receiver trelis.
*******************************************************************************/
int Print_Rx_Trellis(p_state,p_input)
int p_state,p_input;
{
 int cnt;
 printf("\t Rx. trellis Pre. state:%3d Input:%3d ",p_state,p_input);
 printf("Nxt. state:%3d\n",Rx_Trel_NS[p_state][p_input]);
 printf("\t Re. Op:%lf  ",Re_Rx_Trel_Op[p_state][p_input]);
 printf("   Im. Op:%lf\n\n",Im_Rx_Trel_Op[p_state][p_input]);
 return(0);
}
/******************************************************************************/
/*                       Modify the receiver trellis.
*******************************************************************************/
/*int Modify_Rx_Trellis()
{
 int state;
 for(state=ZERO;state<MOD_NUM_RX_STATES;state++)
 {
 }
 return(0);
}*/
/******************************************************************************/
