/******************************************************************************/
/*   Copyright (C) 2001 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "n3type.h"
#include "n3ext.h"
/******************************************************************************/
/*                 Get the transmitter and receiver trellis.
*******************************************************************************/
int Get_Trellis()
{
 Get_Tx_Trellis();
 Get_Rx_Trellis();
 Ck_Mod_State();
}
/******************************************************************************/
/*                      Get the transmitter trellis.
*******************************************************************************/
int Get_Tx_Trellis()
{
 int p_state,p_input;
 for(p_state=ZERO;p_state<NUM_TX_STATES;p_state++)
 {
  for(p_input=ZERO;p_input<NUM_INPUTS;p_input++)
  {
   Tx_Trel_Op[p_state][p_input]=Get_Tx_Trel_Op(p_state,p_input);
   Tx_Trel_NS[p_state][p_input]=(2*p_state+p_input)%NUM_TX_STATES;
   printf("\t Tx. trellis Pre. state:%3d Input:%3d ",p_state,p_input);
   printf("Nxt. state:%3d ",Tx_Trel_NS[p_state][p_input]);
   printf("Op:%3d.\n",Tx_Trel_Op[p_state][p_input]);
  }
 }
 printf("\n");
 return(0);
}
/******************************************************************************/
/*                   Get the output. LSB maps to real part.
*******************************************************************************/
int Get_Tx_Trel_Op(p_state,p_input)
int p_state,p_input;
{
 int bit0,bit1,op0,op1,dec_op;
 bit0=p_state&1;
 bit1=(p_state/2)&1;
 op0=p_input^bit1;
 op1=p_input^bit0^bit1;
 dec_op=op0+2*op1;
 return(dec_op);
}
/******************************************************************************/
/*                      Get the trellis at the receiver.
*******************************************************************************/
int Get_Rx_Trellis()
{
 int p_state,p_input,ns[NUM_RX_STATES][NUM_INPUTS];
 double reop[NUM_RX_STATES][NUM_INPUTS],imop[NUM_RX_STATES][NUM_INPUTS];
 for(p_state=ZERO;p_state<NUM_TX_STATES;p_state++)
 {
  for(p_input=ZERO;p_input<MOD_NUM_INPUTS;p_input++)
   Process_Rx_Trel(p_state,p_input,ns,reop,imop);
 }
 Modify_Rx_Trellis(ns,reop,imop);
 return(0);
}
/******************************************************************************/
/*      Process the receiver trellis for a given encoder state and input.
*******************************************************************************/
int Process_Rx_Trel(p_state,p_input,ns,reop,imop)
int p_state,p_input,ns[NUM_RX_STATES][NUM_INPUTS];
double reop[NUM_RX_STATES][NUM_INPUTS],imop[NUM_RX_STATES][NUM_INPUTS];
{
 int enc_state,chan_state,net_p_state,chan_st_array[CHAN_MEM_SIZE];
 int cnt,inp,ip_st_array[CHAN_SIZE];
 Decimal_To_M_ary(p_input,ip_st_array,CHAN_SIZE,2); /* Index 0 is oldest bit. */
 enc_state=p_state;
 for(cnt=ZERO;cnt<CHAN_MEM_SIZE;cnt++)
 {
  inp=ip_st_array[cnt];
  chan_st_array[CHAN_MEM_SIZE-ONE-cnt]=Tx_Trel_Op[enc_state][inp];
  enc_state=Tx_Trel_NS[enc_state][inp];
 }
 net_p_state=Get_Net_P_State(enc_state,chan_st_array);
 Get_Rx_Trel_Op(net_p_state,enc_state,ip_st_array,chan_st_array,reop,imop);
 Get_Rx_Trel_NS(net_p_state,enc_state,ip_st_array,chan_st_array,ns);
 return(0);
}
/******************************************************************************/
/*                     Get the resultant present state.
*******************************************************************************/
int Get_Net_P_State(enc_state,chan_st_array)
int enc_state,chan_st_array[];
{
 int net_p_state,chan_state;
 chan_state=M_ary_To_Decimal(chan_st_array,CHAN_MEM_SIZE,CONSTELL_SIZE);
 net_p_state=enc_state*NUM_CHAN_STATES+chan_state;
 return(net_p_state);
}
/******************************************************************************/
/* Get the output for the receiver trellis, given the input and the
   current state.
*******************************************************************************/
int Get_Rx_Trel_Op(net_p_state,enc_state,ip_st_array,chan_st_array,reop,imop)
int net_p_state,enc_state,ip_st_array[],chan_st_array[];
double reop[NUM_RX_STATES][NUM_INPUTS],imop[NUM_RX_STATES][NUM_INPUTS];
{
 int cnt,sym,inp;
 double re_sym,im_sym,re_op,im_op;
 inp=ip_st_array[CHAN_SIZE-ONE];                          /* Most recent bit. */
 sym=Tx_Trel_Op[enc_state][inp];
 re_op=Re_Chan_Tap[ZERO]*Re_Map[sym]-Im_Chan_Tap[ZERO]*Im_Map[sym];
 im_op=Re_Chan_Tap[ZERO]*Im_Map[sym]+Im_Chan_Tap[ZERO]*Re_Map[sym];
 for(cnt=ZERO;cnt<CHAN_MEM_SIZE;cnt++)
 {
  sym=chan_st_array[cnt];
  re_op=re_op+Re_Chan_Tap[cnt+1]*Re_Map[sym]-Im_Chan_Tap[cnt+1]*Im_Map[sym];
  im_op=im_op+Re_Chan_Tap[cnt+1]*Im_Map[sym]+Im_Chan_Tap[cnt+1]*Re_Map[sym];
 }
 reop[net_p_state][inp]=re_op;
 imop[net_p_state][inp]=im_op;
 return(0);
}
/******************************************************************************/
/* Get the next state for the receiver trellis, given the input and the
   net current state.
*******************************************************************************/
int Get_Rx_Trel_NS(net_p_state,enc_state,ip_st_array,chan_st_array,ns)
int net_p_state,enc_state,ip_st_array[],chan_st_array[];
int ns[NUM_RX_STATES][NUM_INPUTS];
{
 int inp,cnt,chan_state,enc_op,enc_ns,net_n_state;
 inp=ip_st_array[CHAN_SIZE-ONE];                          /* Most recent bit. */
 enc_op=Tx_Trel_Op[enc_state][inp];
 enc_ns=Tx_Trel_NS[enc_state][inp];
 for(cnt=CHAN_MEM_SIZE-ONE;cnt>ZERO;cnt--)
  chan_st_array[cnt]=chan_st_array[cnt-ONE];
 chan_st_array[ZERO]=enc_op;
 chan_state=M_ary_To_Decimal(chan_st_array,CHAN_MEM_SIZE,CONSTELL_SIZE);
 net_n_state=enc_ns*NUM_CHAN_STATES+chan_state;
 ns[net_p_state][inp]=net_n_state;
 Get_Mod_State_Mapping(net_n_state);
 return(0);
}
/******************************************************************************/
/*                     Get the modified state mapping.
*******************************************************************************/
int Get_Mod_State_Mapping(net_n_state)
int net_n_state;
{
 if(Ck_Map[net_n_state] == FALSE)
 {
  Mod_State_Map[net_n_state]=Mod_State;
  Iv_Mod_State_Map[Mod_State]=net_n_state;
  Ck_Map[net_n_state]=TRUE;
  Mod_State++;
 }
 return(0);
}
/******************************************************************************/
/*                       Modify the receiver trellis.
*******************************************************************************/
int Modify_Rx_Trellis(ns,reop,imop)
int ns[NUM_RX_STATES][NUM_INPUTS];
double reop[NUM_RX_STATES][NUM_INPUTS],imop[NUM_RX_STATES][NUM_INPUTS];
{
 int state,ip,act_state,n_state,act_n_state;
 for(state=ZERO;state<MOD_NUM_RX_STATES;state++)
 {
  act_state=Iv_Mod_State_Map[state];
  for(ip=ZERO;ip<NUM_INPUTS;ip++)
  {
   act_n_state=ns[act_state][ip];
   n_state=Mod_State_Map[act_n_state];
   Rx_Trel_NS[state][ip]=n_state;
   Re_Rx_Trel_Op[state][ip]=reop[act_state][ip];
   Im_Rx_Trel_Op[state][ip]=imop[act_state][ip];
   Print_Rx_Trellis(state,ip);
  }
 }
 return(0);
}
/******************************************************************************/
/*                        Print the receiver trelis.
*******************************************************************************/
int Print_Rx_Trellis(p_state,p_input)
int p_state,p_input;
{
 printf("\t Rx. trellis Pre. state:%3d Input:%3d ",p_state,p_input);
 printf("Nxt. state:%3d\n",Rx_Trel_NS[p_state][p_input]);
 printf("\t Re. Op:%lf  ",Re_Rx_Trel_Op[p_state][p_input]);
 printf("   Im. Op:%lf\n\n",Im_Rx_Trel_Op[p_state][p_input]);
 return(0);
}
/******************************************************************************/
/*                  Check the value of the modified state.
*******************************************************************************/
int Ck_Mod_State()
{
 if(Mod_State != MOD_NUM_RX_STATES)
  Error_Mes(12);
 return(0);
}
/******************************************************************************/
