/******************************************************************************/
/*   Copyright (C) 2001 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "n3type.h"
#include "n3ext.h"
/******************************************************************************/
/*                         Convert decimal to M-ary.
*******************************************************************************/
int Decimal_To_M_ary(state,m_array,size,base)
int state,m_array[],size,base;
{
 int cnt;
 for(cnt=ZERO;cnt<size;cnt++)
  m_array[cnt]=ZERO;
 cnt=ZERO;
 while(state>ZERO)
 {
  if(cnt == size)
   Error_Mes(3);
  m_array[cnt]=state%base;
  state=state/base;
  cnt++;
 }
 return(0);
}
/******************************************************************************/
/*                          Convert M-ary to decimal.
*******************************************************************************/
int M_ary_To_Decimal(m_array,size,base)
int m_array[],size,base;
{
 int cnt,state,power;
 state=ZERO;
 power=ONE;
 for(cnt=ZERO;cnt<size;cnt++)
 {
  state=state+m_array[cnt]*power;
  power=power*base;
 }
 return(state);
}
/******************************************************************************/
/*                Right shift the contents of the registers.
*******************************************************************************/
int Shift_Right(re_array,im_array,size)
int size;
double re_array[],im_array[];
{
 int cnt;
 for(cnt=(size-ONE);cnt>ZERO;cnt--)
 {
  re_array[cnt]=re_array[cnt-ONE];
  im_array[cnt]=im_array[cnt-ONE];
 }
 return(0);
}
/******************************************************************************/
/* Complex convolution. One 'xxip1[]' is real and 'xxip2[]' are complex.
   Input arrays: 'xxip1[ip1size]' is first array.
                 'xxip2[ip2size]' is second array.
   Output array: 'xxop[]'.
*******************************************************************************/
int Convolve(reip1,imip1,reip2,imip2,reop,imop,ip1size,ip2size)
int ip1size,ip2size;
double reip1[],imip1[],reip2[],imip2[],reop[],imop[];
{
 int opcnt,ipcnt,loop;
 for(opcnt=ZERO;opcnt<(ip1size+ip2size-1);opcnt++)
 {
  reop[opcnt]=imop[opcnt]=0.0;
  ipcnt=ZERO;
  loop=TRUE;
  while((ipcnt<ip1size)&&(loop==TRUE))
  {
   if((opcnt-ipcnt)<ZERO)
    loop=FALSE;
   else if((opcnt-ipcnt)>=ip2size)
    ipcnt=opcnt-ip2size+1;
   else
   {
    reop[opcnt]=reop[opcnt]+reip1[ipcnt]*reip2[opcnt-ipcnt];
    reop[opcnt]=reop[opcnt]-imip1[ipcnt]*imip2[opcnt-ipcnt];
    imop[opcnt]=imop[opcnt]+reip1[ipcnt]*imip2[opcnt-ipcnt];
    imop[opcnt]=imop[opcnt]+imip1[ipcnt]*reip2[opcnt-ipcnt];
    ipcnt++;
   }
  }
 }
 return(0);
}
/******************************************************************************/
/*             Copy a sequence in reverse complex conjugate order.
*******************************************************************************/
int Copy_Rev_Conjugate(re_orig,im_orig,re_copy,im_copy,size)
int size;
double re_orig[],im_orig[],re_copy[],im_copy[];
{
 int cnt;
 for(cnt=ZERO;cnt<size;cnt++)
 {
  re_copy[cnt]=  re_orig[size-ONE-cnt];
  im_copy[cnt]= -im_orig[size-ONE-cnt];
 }
 return(0);
}
/******************************************************************************/
/*       Convert uniform random variables to gaussian random variables.
*******************************************************************************/
int My_Gauss(tvar,tnarry)
double tvar,tnarry[];
{
 double r,theta;
 r= -2.0*tvar*log(1.0-tnarry[URADIUS]);
 if(r < FRAC_ZERO)
  Error_Mes(10);
 r=sqrt(r);
 theta=2.0*MY_PI*tnarry[UTHETA];
 tnarry[GRAND_0]=r*cos(theta);
 tnarry[GRAND_1]=r*sin(theta);
 return(0);
}
/******************************************************************************/
/*                 Check for memory allocation failure.
*******************************************************************************/
int Ck_Calloc(tmp)
void *tmp;
{
 if(tmp==NULL)
  exit(3);
 return(0);
}
/******************************************************************************/
/*                             Error message.
*******************************************************************************/
int Error_Mes(num)
int num;
{
 fprintf(stderr,"\t Simulation error. Exiting with code %d.\n\n",num);
 exit(num);
 return(0);
}
/******************************************************************************/
