/******************************************************************************/
/*   Copyright (C) 2001 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include <stdio.h>
#include <math.h>
/******************************************************************************/
#define ORDER 1                                   /* Order of the IIR filter. */
#define IR_SIZE 1000                             /* Size of impulse response. */
#define AC_SIZE (2*IR_SIZE-1)         /* Size of autocor of impulse response. */
#define ZERO 0
#define ONE 1
#define TRUE 1
#define FALSE 0
/******************************************************************************/
FILE *fp1,*fp2;                                             /* File pointers. */
double Ip_Coeff;                                        /* Input coefficient. */
double Coeff_Array[ORDER];                              /* IIR coeff. of o/p. */
double Op_Reg[ORDER];                                     /* Output register. */
double Imp_Res[IR_SIZE];                                 /* Impulse response. */
double Rev_Imp_Res[IR_SIZE];                             /* Impulse response. */
double Auto_Corr[AC_SIZE];                                /* Autocorrelation. */
/******************************************************************************/
main()
{
 Open_Files();
 Get_Coeff();
 Init_Op_Reg();
 Get_IR();
 Get_Autocorr();
 Print_Autocorr();
}
/******************************************************************************/
/*                           Open files for writing.
*******************************************************************************/
int Open_Files()
{
 fp1=fopen("pc1data/impulse.dat","w");
 fp2=fopen("pc1data/autocor.dat","w");
 return(0);
}
/******************************************************************************/
/*                   Get the IIR filter coefficients.
*******************************************************************************/
int Get_Coeff()
{
 double tmp;
 if(ORDER == 3)
 {
  Ip_Coeff=0.008;
  tmp=0.8;
  Coeff_Array[0]= -3.0*tmp;
  Coeff_Array[1]=  3.0*tmp*tmp;
  Coeff_Array[2]= -tmp*tmp*tmp;
 }
 else if(ORDER == 1)
 {
  tmp= -0.8;
  Ip_Coeff=1.0-tmp;
  Coeff_Array[0]= -tmp;
 }
 return(0);
}
/******************************************************************************/
/*                      Initialize the output register.
*******************************************************************************/
int Init_Op_Reg()
{
 int cnt;
 for(cnt=ZERO;cnt<ORDER;cnt++)
  Op_Reg[cnt]=0.0;
 return(0);
}
/******************************************************************************/
/*                         Get the impulse response.
*******************************************************************************/
int Get_IR()
{
 int cnt1,cnt2;
 double inp;
 inp=1.0;
 for(cnt1=ZERO;cnt1<IR_SIZE;cnt1++)
 {
  Imp_Res[cnt1]=Ip_Coeff*inp;
  for(cnt2=ZERO;cnt2<ORDER;cnt2++)
   Imp_Res[cnt1]=Imp_Res[cnt1]-Coeff_Array[cnt2]*Op_Reg[cnt2];
  for(cnt2=ORDER-ONE;cnt2>ZERO;cnt2--)
   Op_Reg[cnt2]=Op_Reg[cnt2-ONE];
  Rev_Imp_Res[IR_SIZE-ONE-cnt1]=Op_Reg[ZERO]=Imp_Res[cnt1];
  fprintf(fp1,"%lf %lf\n",(double)cnt1,Imp_Res[cnt1]);
  inp=0.0;
 }
 return(0);
}
/******************************************************************************/
/*                           Get the autocorrelation.
*******************************************************************************/
int Get_Autocorr()
{
 Convolve(Imp_Res,Rev_Imp_Res,Auto_Corr,IR_SIZE,IR_SIZE);
 /*Normalize();*/
 return(0);
}
/******************************************************************************/
/*                       Normalize the autocorrelation.
*******************************************************************************/
int Normalize()
{
 int cnt;
 double tmp;
 tmp=Auto_Corr[IR_SIZE-ONE];
 for(cnt=ZERO;cnt<AC_SIZE;cnt++)
  Auto_Corr[cnt]=Auto_Corr[cnt]/tmp;
 return(0);
}
/******************************************************************************/
/*                         Print the autocorrelation.
*******************************************************************************/
int Print_Autocorr()
{
 int cnt;
 for(cnt=ZERO;cnt<AC_SIZE;cnt++)
  fprintf(fp2,"%lf %lf\n",(double)cnt,Auto_Corr[cnt]);
 return(0);
}
/******************************************************************************/
/*                               Real convolution.
*******************************************************************************/
int Convolve(ip1,ip2,op,ip1size,ip2size)
int ip1size,ip2size;
double ip1[],ip2[],op[];
{
 int opcnt,ipcnt,loop;
 for(opcnt=ZERO;opcnt<(ip1size+ip2size-1);opcnt++)
 {
  op[opcnt]=0.0;
  ipcnt=ZERO;
  loop=TRUE;
  while((ipcnt<ip1size)&&(loop==TRUE))
  {
   if((opcnt-ipcnt)<ZERO)
    loop=FALSE;
   else if((opcnt-ipcnt)>=ip2size)
    ipcnt=opcnt-ip2size+1;
   else
   {
    op[opcnt]=op[opcnt]+ip1[ipcnt]*ip2[opcnt-ipcnt];
    ipcnt++;
   }
  }
 }
 return(0);
}
/******************************************************************************/
