/******************************************************************************/
/*   Copyright (C) 2001 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "pc10type.h"
#include "pc10ext.h"
/******************************************************************************/
/*             Update equalizer taps in decision directed mode.
*******************************************************************************/
int DD_Update_Eq_Taps()
{
 if(DD_MODE)
  Compute_DD_Eq_Op_Error();
 else
  Compute_CD_Eq_Op_Error();
 Compute_DD_Eq_Input_Energy();
 Update_DD_Equalizer_Taps();
 return(0);
}
/******************************************************************************/
/* Compute equalizer output error in decision directed mode (estimated
*  decisions).
*******************************************************************************/
int Compute_DD_Eq_Op_Error()
{
 int rx_reg_inx;
 double re_sym,im_sym,err_sq;
 re_sym=Re_Map[VA_Op_Sym_Num];
 im_sym=Im_Map[VA_Op_Sym_Num];
 rx_reg_inx=VA_DEPTH-ONE;
 Re_Eq_Err=re_sym-Re_Eq_Op_Reg[rx_reg_inx];
 Im_Eq_Err=im_sym-Im_Eq_Op_Reg[rx_reg_inx];
 err_sq=Re_Eq_Err*Re_Eq_Err+Im_Eq_Err*Im_Eq_Err;
// printf("DD re:%lf dre:%lf\n",re_sym,Re_Eq_Op_Reg[rx_reg_inx]);
 if(Sym_Cnt > 9999000)
  fprintf(fp3,"%lf\n",err_sq);
 return(0);
}
/******************************************************************************/
/*         Compute equalizer output error using correct decisions.
*******************************************************************************/
int Compute_CD_Eq_Op_Error()
{
 int sym_num,rx_reg_inx,buf_inx;
 double re_sym,im_sym,err_sq;
 buf_inx=Eq_Err_Buf_Inx-MAX_SYS_DELAY-ONE;
 if(buf_inx < ZERO)
  Error_Mes(9);
 sym_num=Eq_Err_Buf[buf_inx];
 re_sym=Re_Map[sym_num];
 im_sym=Im_Map[sym_num];
 rx_reg_inx=VA_DEPTH-ONE;
 Re_Eq_Err=re_sym-Re_Eq_Op_Reg[rx_reg_inx];
 Im_Eq_Err=im_sym-Im_Eq_Op_Reg[rx_reg_inx];
 err_sq=Re_Eq_Err*Re_Eq_Err+Im_Eq_Err*Im_Eq_Err;
// printf("sym:%d DD re:%lf eqre:%lf ",Sym_Cnt,re_sym,Re_Eq_Op_Reg[rx_reg_inx]);
// Get_Eq_Op_Tmp();
 if(Sym_Cnt > 9999000)
  fprintf(fp3,"%lf\n",err_sq);
 return(0);
}
/******************************************************************************/
/*                          Get equalizer output.
*******************************************************************************/
int Get_Eq_Op_Tmp()
{
 int cnt,inx;
 double re_eq_op,im_eq_op;
 re_eq_op=im_eq_op=0.0;
 for(cnt=ZERO;cnt<EQ_TAP_SIZE;cnt++)
 {
  inx=SAMP_PER_SYM*(VA_DEPTH-ONE)+cnt;
  if(inx >= EQ_REG_SIZE)
   Error_Mes(9);
  re_eq_op=re_eq_op+Re_Eq_Reg[inx]*Re_Eq_Tap[cnt];
  re_eq_op=re_eq_op-Im_Eq_Reg[inx]*Im_Eq_Tap[cnt];
  im_eq_op=im_eq_op+Re_Eq_Reg[inx]*Im_Eq_Tap[cnt];
  im_eq_op=im_eq_op+Im_Eq_Reg[inx]*Re_Eq_Tap[cnt];
 }
 printf("ceqre:%lf\n",re_eq_op);
 fflush(stdout);
 return(0);
}
/******************************************************************************/
/*         Compute equalizer input energy in decision directed mode.
*******************************************************************************/
int Compute_DD_Eq_Input_Energy()
{
 int cnt,inx;
 Eq_Inp_Energy=0.0;
 for(cnt=ZERO;cnt<EQ_TAP_SIZE;cnt++)
 {
  inx=SAMP_PER_SYM*(VA_DEPTH-ONE)+cnt;
  if(inx >= EQ_REG_SIZE)
   Error_Mes(9);
  Eq_Inp_Energy=Eq_Inp_Energy+Re_Eq_Reg[inx]*Re_Eq_Reg[inx];
  Eq_Inp_Energy=Eq_Inp_Energy+Im_Eq_Reg[inx]*Im_Eq_Reg[inx];
 }
 return(0);
}
/******************************************************************************/
/*             Update equalizer taps in decision directed mode.
*******************************************************************************/
int Update_DD_Equalizer_Taps()
{
 int cnt,inx;
 double re_grad,im_grad,ss_fac;
 ss_fac=0.01;
 for(cnt=ZERO;cnt<EQ_TAP_SIZE;cnt++)
 {
  inx=SAMP_PER_SYM*(VA_DEPTH-ONE)+cnt;
  if(inx >= EQ_REG_SIZE)
   Error_Mes(9);
  re_grad=Re_Eq_Err*Re_Eq_Reg[inx]+Im_Eq_Err*Im_Eq_Reg[inx];
  im_grad=Im_Eq_Err*Re_Eq_Reg[inx]-Re_Eq_Err*Im_Eq_Reg[inx];
  Re_Eq_Tap[cnt]=Re_Eq_Tap[cnt]+ss_fac*EQ_STEP*re_grad/Eq_Inp_Energy;
  Im_Eq_Tap[cnt]=Im_Eq_Tap[cnt]+ss_fac*EQ_STEP*im_grad/Eq_Inp_Energy;
 }
 return(0);
}
/******************************************************************************/
