/******************************************************************************/
/*   Copyright (C) 2001 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "pc10type.h"
#include "pc10ext.h"
/******************************************************************************/
int Simulation()
{
 for(Sym_Cnt=ZERO;Sym_Cnt<MAX_SYM;Sym_Cnt++)
 {
  Get_Symbol();
  for(Samp_Cnt=ZERO;Samp_Cnt<SAMP_PER_SYM;Samp_Cnt++)
  {
   Get_Channel();
   Get_Channel_Output();
   Add_Noise();
   Receive();
  }
 }
 return(0);
}
/******************************************************************************/
/*                          Get the modified channel.
*******************************************************************************/
int Get_Channel()
{
 if(Fade_Samp_Cnt[Samp_Cnt] == ZERO)
  Shift_In_New_Noise();
 Get_Fade_Noise();
 Get_New_Channel();
 Fade_Samp_Cnt[Samp_Cnt]++;
 if(Fade_Samp_Cnt[Samp_Cnt] == IP_FAC)
  Fade_Samp_Cnt[Samp_Cnt]=ZERO;
 return(0);
}
/******************************************************************************/
/*                Shift in new noise samples into the registers.
*******************************************************************************/
int Shift_In_New_Noise()
{
 int cnt1,cnt2;
 for(cnt1=Samp_Cnt;cnt1<CHAN_SIZE;cnt1=cnt1+SAMP_PER_SYM)
 {
  for(cnt2=FF_REG_SIZE-ONE;cnt2>ZERO;cnt2--)
   Fade_Reg[cnt1][cnt2]=Fade_Reg[cnt1][cnt2-ONE];
 }
 Shift_In_AWGN();
 return(0);
}
/******************************************************************************/
/*                              Shift in AWGN.
*******************************************************************************/
int Shift_In_AWGN()
{
 int cnt1;
 double erand48();
 for(cnt1=Samp_Cnt;cnt1<CHAN_SIZE;cnt1=cnt1+TWO*SAMP_PER_SYM)
 {
  N_Val[URADIUS]=erand48(N1_Store);
  N_Val[UTHETA]=erand48(N2_Store);
  My_Gauss(Fade_Var,N_Val);
  Fade_Reg[cnt1][ZERO]=N_Val[GRAND_0];
  if((cnt1+SAMP_PER_SYM) < CHAN_SIZE)
   Fade_Reg[cnt1+SAMP_PER_SYM][ZERO]=N_Val[GRAND_1];
 }
 return(0);
}
/******************************************************************************/
/*       Get the correlated fade noise at the output of the fade filter.
*******************************************************************************/
int Get_Fade_Noise()
{
 int cnt1,cnt2,inx;
 for(cnt1=Samp_Cnt;cnt1<CHAN_SIZE;cnt1=cnt1+SAMP_PER_SYM)
 {
  Fade_Op[cnt1]=0.0;
  inx=ZERO;
  for(cnt2=Fade_Samp_Cnt[Samp_Cnt];cnt2<FF_SIZE;cnt2=cnt2+IP_FAC)
  {
   if(inx < FF_REG_SIZE)
    Fade_Op[cnt1]=Fade_Op[cnt1]+F_Filter[cnt2]*Fade_Reg[cnt1][inx];
   inx++;
  }
 }
 return(0);
}
/******************************************************************************/
/*                         Get the new channel taps.
*******************************************************************************/
int Get_New_Channel()
{
 int cnt1;
 for(cnt1=Samp_Cnt;cnt1<CHAN_SIZE;cnt1=cnt1+SAMP_PER_SYM)
 {
  F_Chan_Tap[cnt1]=Chan_Tap[cnt1]+Fade_Op[cnt1];
//  if(Sym_Cnt < 1000)
//   printf("cnt1:%d fade:%lf\n",cnt1,Fade_Op[cnt1]);
 }
 return(0);
}
/******************************************************************************/
/*                        Get the channel output.
*******************************************************************************/
int Get_Channel_Output()
{
 int cnt;
 Shift_Right(Re_Chan_Reg,Im_Chan_Reg,CHAN_SIZE);
 Re_Chan_Reg[ZERO]=Re_Sym;
 Im_Chan_Reg[ZERO]=Im_Sym;
 Re_Chan_Op=Im_Chan_Op=0.0;
 for(cnt=Samp_Cnt;cnt<CHAN_SIZE;cnt=cnt+SAMP_PER_SYM)
 {
  Re_Chan_Op=Re_Chan_Op+Re_Chan_Reg[cnt]*F_Chan_Tap[cnt];
  Im_Chan_Op=Im_Chan_Op+Im_Chan_Reg[cnt]*F_Chan_Tap[cnt];
 }
 Re_Sym=Im_Sym=0.0;
 return(0);
}
/******************************************************************************/
/*                        Add white Gaussian noise.
*******************************************************************************/
int Add_Noise()
{
 double erand48();
 N_Val[URADIUS]=erand48(N1_Store);
 N_Val[UTHETA]=erand48(N2_Store);
 My_Gauss(VARIANCE,N_Val);
 Re_Noise=N_Val[GRAND_0];
 Im_Noise=N_Val[GRAND_1];
 Re_Chan_Op=Re_Chan_Op+Re_Noise;
 Im_Chan_Op=Im_Chan_Op+Im_Noise;
 return(0);
}
/******************************************************************************/
/*                         Receiver routines.
*******************************************************************************/
int Receive()
{
 Shift_Into_Equalizer();
 if((Sym_Cnt >= MAX_SYS_DELAY) && (Samp_Cnt == ZERO))
 {
  Get_FF_Eq_Op();
  if(Sym_Cnt <= TOT_TRN_LEN)
   Equalizer_Trn_Procedures();
  Shift_Into_Eq_Op_Reg();
  if(Sym_Cnt >= VA_ON_TIME)
   Viterbi();
  if(Sym_Cnt > TOT_TRN_LEN)
   Steady_State_Process();
  Update_Buffer_Procedures();
 }
 return(0);
}
/******************************************************************************/
/*               Shift the received sample into the equalizer.
*******************************************************************************/
int Shift_Into_Equalizer()
{
 Shift_Right(Re_Eq_Reg,Im_Eq_Reg,EQ_REG_SIZE);
 Re_Eq_Reg[ZERO]=Re_Chan_Op;
 Im_Eq_Reg[ZERO]=Im_Chan_Op;
 return(0);
}
/******************************************************************************/
/*                  Get the feedforward equalizer output.
*******************************************************************************/
int Get_FF_Eq_Op()
{
 int cnt;
 Re_Eq_Op=Im_Eq_Op=0.0;
 for(cnt=ZERO;cnt<EQ_TAP_SIZE;cnt++)
 {
  Re_Eq_Op=Re_Eq_Op+Re_Eq_Reg[cnt]*Re_Eq_Tap[cnt];
  Re_Eq_Op=Re_Eq_Op-Im_Eq_Reg[cnt]*Im_Eq_Tap[cnt];
  Im_Eq_Op=Im_Eq_Op+Re_Eq_Reg[cnt]*Im_Eq_Tap[cnt];
  Im_Eq_Op=Im_Eq_Op+Im_Eq_Reg[cnt]*Re_Eq_Tap[cnt];
 }
 return(0);
}
/******************************************************************************/
/*    Shift the equalizer output into the register for processing by VA.
*******************************************************************************/
int Shift_Into_Eq_Op_Reg()
{
 int cnt;
 for(cnt=EQ_OP_REG_SIZE-ONE;cnt>ZERO;cnt--)
 {
  Re_Eq_Op_Reg[cnt]=Re_Eq_Op_Reg[cnt-ONE];
  Im_Eq_Op_Reg[cnt]=Im_Eq_Op_Reg[cnt-ONE];
 }
 Re_Eq_Op_Reg[ZERO]=Re_Eq_Op;
 Im_Eq_Op_Reg[ZERO]=Im_Eq_Op;
 return(0);
}
/******************************************************************************/
/*                       Do steady-state processing.
*******************************************************************************/
int Steady_State_Process()
{
 Shift_In_VA_Op();
 DD_Update_Eq_Taps();
// if(Sym_Cnt > PRED_UP_START)
//  DD_Pred_Update();
 Update_Error();
 return(0);
}
/******************************************************************************/
/*                    Store the VA output into a register.
*******************************************************************************/
int Shift_In_VA_Op()
{
 int cnt;
 for(cnt=VA_OP_REG_SIZE-ONE;cnt>ZERO;cnt--)
  VA_Op_Reg[cnt]=VA_Op_Reg[cnt-ONE];
 VA_Op_Reg[ZERO]=VA_Op_Sym_Num;
 return(0);
}
/******************************************************************************/
/*                     Update the symbol error count.
*******************************************************************************/
int Update_Error()
{
 int cnt,inx;
 inx=Err_Buf_Inx-MAX_SYS_DELAY-ONE;
 if(inx < ZERO)
  Error_Mes(9);
// if(Sym_Cnt > 9000)
//  printf("cnt:%d ebuf:%d va:%d\n",Sym_Cnt,Err_Buf[inx],VA_Ip_Sym_Num);
 if(Err_Buf[inx] != VA_Ip_Sym_Num)
  Err_Cnt++;
 Valid_Sym++;
 return(0);
}
/******************************************************************************/
/*               Procedures for updating the error buffers.
*******************************************************************************/
int Update_Buffer_Procedures()
{
 Update_Eq_Err_Buf();
 Update_Sym_Error_Buf();
 return(0);
}
/******************************************************************************/
/*                     Update the equalizer error buffer.
*******************************************************************************/
int Update_Eq_Err_Buf()
{
 int cnt;
 for(cnt=ZERO;cnt<=(Eq_Err_Buf_Inx-2);cnt++)
  Eq_Err_Buf[cnt]=Eq_Err_Buf[cnt+ONE];
 Eq_Err_Buf_Inx--;
 if(Eq_Err_Buf_Inx < ZERO)
  Error_Mes(7);
 return(0);
}
/******************************************************************************/
/*                     Update the symbol error buffer.
*******************************************************************************/
int Update_Sym_Error_Buf()
{
 int cnt;
 for(cnt=ZERO;cnt<=(Err_Buf_Inx-2);cnt++)
  Err_Buf[cnt]=Err_Buf[cnt+ONE];
 Err_Buf_Inx--;
 if(Err_Buf_Inx < ZERO)
  Error_Mes(7);
 return(0);
}
/******************************************************************************/
