/******************************************************************************/
/*   Copyright (C) 2001 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "pc11type.h"
#include "pc11ext.h"
/******************************************************************************/
int Simulation()
{
 for(Sym_Cnt=ZERO;Sym_Cnt<MAX_SYM;Sym_Cnt++)
 {
  Get_Symbol();
  for(Samp_Cnt=ZERO;Samp_Cnt<SAMP_PER_SYM;Samp_Cnt++)
  {
   Get_Channel_Output();
   Add_Noise();
   Receive();
  }
 }
 return(0);
}
/******************************************************************************/
/*                        Get the channel output.
*******************************************************************************/
int Get_Channel_Output()
{
 int cnt;
 Shift_Right(Re_Chan_Reg,Im_Chan_Reg,CHAN_SIZE);
 Re_Chan_Reg[ZERO]=Re_Sym;
 Im_Chan_Reg[ZERO]=Im_Sym;
 Re_Chan_Op=Im_Chan_Op=0.0;
 for(cnt=Samp_Cnt;cnt<CHAN_SIZE;cnt=cnt+SAMP_PER_SYM)
 {
  Re_Chan_Op=Re_Chan_Op+Re_Chan_Reg[cnt]*Re_Chan_Tap[cnt];
  Re_Chan_Op=Re_Chan_Op-Im_Chan_Reg[cnt]*Im_Chan_Tap[cnt];
  Im_Chan_Op=Im_Chan_Op+Re_Chan_Reg[cnt]*Im_Chan_Tap[cnt];
  Im_Chan_Op=Im_Chan_Op+Im_Chan_Reg[cnt]*Re_Chan_Tap[cnt];
 }
 Re_Sym=Im_Sym=0.0;
 return(0);
}
/******************************************************************************/
/*                        Add white Gaussian noise.
*******************************************************************************/
int Add_Noise()
{
 double erand48();
 N_Val[URADIUS]=erand48(N1_Store);
 N_Val[UTHETA]=erand48(N2_Store);
 My_Gauss(VARIANCE,N_Val);
 Re_Noise=N_Val[GRAND_0];
 Im_Noise=N_Val[GRAND_1];
 Re_Chan_Op=Re_Chan_Op+Re_Noise;
 Im_Chan_Op=Im_Chan_Op+Im_Noise;
 return(0);
}
/******************************************************************************/
/*                         Receiver routines.
*******************************************************************************/
int Receive()
{
 Shift_Into_Equalizer();
 if((Sym_Cnt >= MAX_SYS_DELAY) && (Samp_Cnt == ZERO))
 {
  Get_FF_Eq_Op();
  Get_FB_Eq_Op();
  if(Sym_Cnt <= TOT_TRN_LEN)
   Equalizer_Trn_Procedures();
  Shift_Into_Eq_Op_Reg();
  if(Sym_Cnt >= VA_ON_TIME)
   Viterbi();
  if(Sym_Cnt > TOT_TRN_LEN)
   Steady_State_Process();
  Update_Buffer_Procedures();
 }
 return(0);
}
/******************************************************************************/
/*               Shift the received sample into the equalizer.
*******************************************************************************/
int Shift_Into_Equalizer()
{
 Shift_Right(Re_Eq_Reg,Im_Eq_Reg,EQ_REG_SIZE);
 Re_Eq_Reg[ZERO]=Re_Chan_Op;
 Im_Eq_Reg[ZERO]=Im_Chan_Op;
 return(0);
}
/******************************************************************************/
/*                  Get the feedforward equalizer output.
*******************************************************************************/
int Get_FF_Eq_Op()
{
 int cnt;
 Re_Eq_Op=Im_Eq_Op=0.0;
 for(cnt=ZERO;cnt<EQ_TAP_SIZE;cnt++)
 {
  Re_Eq_Op=Re_Eq_Op+Re_Eq_Reg[cnt]*Re_Eq_Tap[cnt];
  Re_Eq_Op=Re_Eq_Op-Im_Eq_Reg[cnt]*Im_Eq_Tap[cnt];
  Im_Eq_Op=Im_Eq_Op+Re_Eq_Reg[cnt]*Im_Eq_Tap[cnt];
  Im_Eq_Op=Im_Eq_Op+Im_Eq_Reg[cnt]*Re_Eq_Tap[cnt];
 }
 return(0);
}
/******************************************************************************/
/*                  Get the feedback equalizer output.
*******************************************************************************/
int Get_FB_Eq_Op()
{
 if(Sym_Cnt <= TOT_TRN_LEN)
  FB_Eq_Op_Trn();                    /* Training mode: use correct decisions. */
 else
 {
  if(DD_MODE)
   FB_Eq_Op_DD();               /* Use estimated decisions to compute eq. op. */
  else
   FB_Eq_Op_Trn();                /* Use correct decisions to compute eq. op. */
 }
 return(0);
}
/******************************************************************************/
/*        Get the feedback equalizer output using correct decisions.
*******************************************************************************/
int FB_Eq_Op_Trn()
{
 int cnt,inx,sym_num;
 double re_sym,im_sym;
 for(cnt=ZERO;cnt<DF_SIZE;cnt++)
 {
  inx=Eq_Err_Buf_Inx-EQ_OP_DELAY-ONE-ONE-cnt;
  if(inx < ZERO)
   Error_Mes(9);
  sym_num=Eq_Err_Buf[inx];
  re_sym=Re_Map[sym_num];
  im_sym=Im_Map[sym_num];
  Re_Eq_Op=Re_Eq_Op+re_sym*Re_DF_Tap[cnt];
  Re_Eq_Op=Re_Eq_Op-im_sym*Im_DF_Tap[cnt];
  Im_Eq_Op=Im_Eq_Op+re_sym*Im_DF_Tap[cnt];
  Im_Eq_Op=Im_Eq_Op+im_sym*Re_DF_Tap[cnt];
 }
 for(cnt=ZERO;cnt<NUM_TX_STATES;cnt++)
 {
  Mod_Re_Eq_Op[cnt]=Re_Eq_Op;
  Mod_Im_Eq_Op[cnt]=Im_Eq_Op;
 }
 return(0);
}
/******************************************************************************/
/*      Get the feedback equalizer output using per survivor processing.
*******************************************************************************/
int FB_Eq_Op_DD()
{
 int cnt,sym_num,inx,cnt1,tstate,tip,top;
 struct Trellis *curr;
 for(cnt1=ZERO;cnt1<NUM_TX_STATES;cnt1++)
 {
  curr=Lst_Depth->Prv_Depth;
  tstate=cnt1;
  Mod_Re_Eq_Op[cnt1]=Re_Eq_Op;
  Mod_Im_Eq_Op[cnt1]=Im_Eq_Op;
  for(cnt=ZERO;cnt<DF_SIZE;cnt++)
  {
   if(curr == NULL)
    Error_Mes(13);
   tip=curr->Ip_Path[tstate];
   tstate=curr->St_Path[tstate];
   curr=curr->Prv_Depth;
   top=Tx_Trel_Op[tstate][tip];
   Mod_Re_Eq_Op[cnt1]=Mod_Re_Eq_Op[cnt1]+Re_Map[top]*Re_DF_Tap[cnt];
   Mod_Re_Eq_Op[cnt1]=Mod_Re_Eq_Op[cnt1]-Im_Map[top]*Im_DF_Tap[cnt];
   Mod_Im_Eq_Op[cnt1]=Mod_Im_Eq_Op[cnt1]+Re_Map[top]*Im_DF_Tap[cnt];
   Mod_Im_Eq_Op[cnt1]=Mod_Im_Eq_Op[cnt1]+Im_Map[top]*Re_DF_Tap[cnt];
  }
 }
 return(0);
}
/******************************************************************************/
/*    Shift the equalizer output into the register for processing by VA.
*******************************************************************************/
int Shift_Into_Eq_Op_Reg()
{
 int cnt,cnt1;
 for(cnt1=ZERO;cnt1<NUM_TX_STATES;cnt1++)
 {
  for(cnt=EQ_OP_REG_SIZE-ONE;cnt>ZERO;cnt--)
  {
   Re_Eq_Op_Reg[cnt1][cnt]=Re_Eq_Op_Reg[cnt1][cnt-ONE];
   Im_Eq_Op_Reg[cnt1][cnt]=Im_Eq_Op_Reg[cnt1][cnt-ONE];
  }
  Re_Eq_Op_Reg[cnt1][ZERO]=Mod_Re_Eq_Op[cnt1];
  Im_Eq_Op_Reg[cnt1][ZERO]=Mod_Im_Eq_Op[cnt1];
 }
 return(0);
}
/******************************************************************************/
/*                       Do steady-state processing.
*******************************************************************************/
int Steady_State_Process()
{
 Shift_In_VA_Op();
 DD_Update_Eq_Taps();
 Update_Error();
 return(0);
}
/******************************************************************************/
/*                    Store the VA output into a register.
*******************************************************************************/
int Shift_In_VA_Op()
{
 int cnt;
 for(cnt=VA_OP_REG_SIZE-ONE;cnt>ZERO;cnt--)
  VA_Op_Reg[cnt]=VA_Op_Reg[cnt-ONE];
 VA_Op_Reg[ZERO]=VA_Op_Sym_Num;
 return(0);
}
/******************************************************************************/
/*                     Update the symbol error count.
*******************************************************************************/
int Update_Error()
{
 int cnt,inx;
 inx=Err_Buf_Inx-VA_OP_DELAY-ONE;
 if(inx < ZERO)
  Error_Mes(9);
// if(Sym_Cnt > 9000)
// printf("cnt:%d ebuf:%d va:%d\n",Sym_Cnt,Err_Buf[inx],VA_Ip_Sym_Num);
 if(Err_Buf[inx] != VA_Ip_Sym_Num)
  Err_Cnt++;
 Valid_Sym++;
 return(0);
}
/******************************************************************************/
/*               Procedures for updating the error buffers.
*******************************************************************************/
int Update_Buffer_Procedures()
{
 Update_Eq_Err_Buf();
 Update_Sym_Error_Buf();
 return(0);
}
/******************************************************************************/
/*                     Update the equalizer error buffer.
*******************************************************************************/
int Update_Eq_Err_Buf()
{
 int cnt;
 for(cnt=ZERO;cnt<=(Eq_Err_Buf_Inx-2);cnt++)
  Eq_Err_Buf[cnt]=Eq_Err_Buf[cnt+ONE];
 Eq_Err_Buf_Inx--;
 if(Eq_Err_Buf_Inx < ZERO)
  Error_Mes(7);
 return(0);
}
/******************************************************************************/
/*                     Update the symbol error buffer.
*******************************************************************************/
int Update_Sym_Error_Buf()
{
 int cnt;
 for(cnt=ZERO;cnt<=(Err_Buf_Inx-2);cnt++)
  Err_Buf[cnt]=Err_Buf[cnt+ONE];
 Err_Buf_Inx--;
 if(Err_Buf_Inx < ZERO)
  Error_Mes(7);
 return(0);
}
/******************************************************************************/
