/******************************************************************************/
/*   Copyright (C) 2001 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "pc12type.h"
#include "pc12ext.h"
/******************************************************************************/
/*                      Get the fade filter coefficients.
*******************************************************************************/
Get_Fade_Filter()
{
 Get_Gamma_1_by_4();
 Get_FF();
 Normalize_FF();
 Get_Auto_Corr();
 Initialize_Fade_Var();
 Initialize_Fade_Reg();
 Initialize_Fade_Samp_Cnt();
 return(0);
}
/******************************************************************************/
/*                             Get Gamma(1/4).
*******************************************************************************/
int Get_Gamma_1_by_4()
{
 int inx;
 double term,numer,denom,cnt;
 Gamma_1b4=16.0*MY_PI*MY_PI;
 for(inx=ONE;inx<=GAMMA_1B4_ITER;inx++)
 {
  cnt=(double)inx;
  numer=(4.0*cnt-1.0)*(4.0*cnt-1.0)*((4.0*cnt+1.0)*(4.0*cnt+1.0)-1.0);
  denom=((4.0*cnt-1.0)*(4.0*cnt-1.0)-1.0)*(4.0*cnt+1.0)*(4.0*cnt+1.0);
  term=numer/denom;
  Gamma_1b4=Gamma_1b4*term;
 }
 Gamma_1b4=sqrt(Gamma_1b4);
 Gamma_1b4=sqrt(Gamma_1b4);
 printf("\t Gamma(1/4):%lf\n\n",Gamma_1b4);
 return(0);
}
/******************************************************************************/
/*                             Get the fade filter.
*******************************************************************************/
int Get_FF()
{
 int cnt;
 double time,fourth_root;
 fourth_root=sqrt(sqrt(MY_PI*F_D));
 for(cnt=ZERO;cnt<FF_SIZE;cnt++)
 {
  time=((double)cnt-FF_DELAY)/F_S;
  if((time > ERROR) || (time < -ERROR))
  {
   Compute_J_1b4(time);
   F_Filter[cnt]=J_b*fourth_root;
  }
  else
   Time_Zero_Process(cnt);
 }
 return(0);
}
/******************************************************************************/
/*     Compute the fractional order Bessel function for the given time.
*******************************************************************************/
int Compute_J_1b4(time)
double time;
{
 int cnt,sign;
 long double term,x,numer,denom,y;
 sign=MINUSONE;
 cnt=ONE;
 x=2.0*MY_PI*F_D*(long double)time;
 numer=x*x;
 y=term=4.0/(long double)Gamma_1b4;
 while(term > ERROR)
 {
  denom=(long double)(4*cnt)*((long double)cnt+0.25);
  term=(term/denom)*numer;
  y=y+(long double)sign*term;
  sign= -sign;
  cnt++;
 }
 J_b=(double)y;
 return(0);
}
/******************************************************************************/
/*           Process the sample at time zero using l'Hospital's rule.
*******************************************************************************/
int Time_Zero_Process(cnt)
int cnt;
{
 double numer,denom;
 numer=sqrt(MY_PI*F_D);
 numer=sqrt(numer);
 denom=0.25*Gamma_1b4;
 F_Filter[cnt]=numer/denom;
 return(0);
}
/******************************************************************************/
/*                  Normalize the fade filter to unit energy.
*******************************************************************************/
int Normalize_FF()
{
 int cnt;
 double energy;
 FILE *fp;
 fp=fopen("pc12data/ffilter.dat","w");
 energy=0.0;
 for(cnt=ZERO;cnt<FF_SIZE;cnt++)
  energy=energy+F_Filter[cnt]*F_Filter[cnt];
 for(cnt=ZERO;cnt<FF_SIZE;cnt++)
 {
  F_Filter[cnt]=F_Filter[cnt]/sqrt(energy);
  fprintf(fp,"%lf %14.10lf\n",(double)cnt,F_Filter[cnt]);
 }
 fclose(fp);
 return(0);
}
/******************************************************************************/
/*   Get the autocorrelation of the fade filter.
*******************************************************************************/
int Get_Auto_Corr()
{
 int cnt;
 double im[FF_SIZE],reop[2*FF_SIZE-1],imop[2*FF_SIZE-1],x;
 FILE *fp1,*fp2;
 fp1=fopen("pc12data/ff_ac.dat","w");
 fp2=fopen("pc12data/j0.dat","w");
 for(cnt=ZERO;cnt<FF_SIZE;cnt++)
  im[cnt]=0.0;
 Convolve(F_Filter,im,F_Filter,im,reop,imop,FF_SIZE,FF_SIZE);
 for(cnt=ZERO;cnt<TWO*FF_SIZE-ONE;cnt++)
 {
  x=2.0*MY_PI*F_D*(double)(cnt-FF_SIZE+ONE)*F_S;
  fprintf(fp1,"%lf %lf\n",(double)cnt,reop[cnt]);
  fprintf(fp2,"%lf %lf\n",(double)cnt,j0(x));
 }
 fclose(fp1);
 fclose(fp2);
 return(0);
}
/******************************************************************************/
/*                      Initialize the fade variance.
*******************************************************************************/
int Initialize_Fade_Var()
{
 if(FADE)
  Fade_Var=1.0/(11.0*(double)CHAN_SIZE);
 else
  Fade_Var=0.0;
 return(0);
}
/******************************************************************************/
/*                     Initialize the fade registers.
*******************************************************************************/
int Initialize_Fade_Reg()
{
 int cnt1,cnt2;
 double erand48();
 for(cnt1=ZERO;cnt1<CHAN_SIZE;cnt1++)
 {
  for(cnt2=ZERO;cnt2<FF_REG_SIZE;cnt2=cnt2+TWO)
  {
   N_Val[URADIUS]=erand48(N1_Store);
   N_Val[UTHETA]=erand48(N2_Store);
   My_Gauss(Fade_Var,N_Val);
   Fade_Reg[cnt1][cnt2]=N_Val[GRAND_0];
   if((cnt2+ONE) < FF_REG_SIZE)
    Fade_Reg[cnt1][cnt2+ONE]=N_Val[GRAND_1];
  }
 }
 return(0);
}
/******************************************************************************/
/*                  Initialize the interpolation counter.
*******************************************************************************/
int Initialize_Fade_Samp_Cnt()
{
 int cnt;
 for(cnt=ZERO;cnt<SAMP_PER_SYM;cnt++)
  Fade_Samp_Cnt[cnt]=ZERO;
 return(0);
}
/******************************************************************************/
