/******************************************************************************/
/*   Copyright (C) 2001 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
/******************************************************************************/
/*                    SIMULATION RUN LENGTH PARAMETER.
*******************************************************************************/
#define VA_DELAY 40                                       /* Delay in the VA. */
#define MAX_SYM (10000000+TOT_TRN_LEN+1)         /* Length of simulation run. */
/******************************************************************************/
/*                         CONSTELLATION PARAMETERS.
*******************************************************************************/
#define CONSTELL_SIZE 4                             /* Size of constellation. */
#define SAMP_PER_SYM 2                                 /* Samples per symbol. */
/******************************************************************************/
/*                          TRELLIS PARAMETERS.
*******************************************************************************/
#define NUM_TX_STATES 4              /* No. of states in transmitter trellis. */
#define NUM_INPUTS 2                                        /* No. of inputs. */
/******************************************************************************/
/*                          CHANNEL PARAMETERS.
*******************************************************************************/
#define CHAN_SIZE 12                                  /* No. of channel taps. */
/******************************************************************************/
/*                           FADE FILTER PARAMETERS.
*******************************************************************************/
#define FADE 1                               /* 1:Fading channel;0:otherwise. */
#define IP_FAC 10                                    /* Interpolation factor. */
#define GAMMA_1B4_ITER 100        /* No. of iterations to compute Gamma(1/4). */
#define BESSEL_ITER 40   /* No. of iterations to compute the Bessel function. */
#define F_S 1.0                                        /* Sampling frequency. */
#define F_D (0.01*F_S)                                  /* Doppler frequency. */
#define FF_DELAY 700                             /* Delay of the fade filter. */
#define FF_SIZE (2*FF_DELAY+ONE)                  /* Size of the fade filter. */
#define FF_REG_SIZE ((int)(FF_SIZE/IP_FAC)+ONE)          /* Size of fade reg. */
/******************************************************************************/
/*                  GAUSSIAN NOISE GENERATOR PARAMETERS.
*******************************************************************************/
#define VARIANCE 00.08                                   /* Noise variance. */
#define PN_SIZE 3                 /* Size of array which stores the PN value. */
#define NVALSIZE 4
#define URADIUS 0
#define UTHETA 1
#define GRAND_0 2
#define GRAND_1 3
/******************************************************************************/
/*                          EQUALIZER PARAMETERS.
*******************************************************************************/
#define DD_MODE 1      /* Tap update: 1:Decision directed; 0:correct symbols. */
#define EQ_TAP_SIZE 51            /* No. of equalizer taps; feedforward part. */
#define DF_SIZE 6                        /* Length of decision feedback part. */
#define EXTRA_EQ_REG_LEN (SAMP_PER_SYM*(VA_DEPTH-1))         /* Extra memory. */
#define EQ_REG_SIZE (EQ_TAP_SIZE+EXTRA_EQ_REG_LEN)  /* Size of equalizer reg. */
#define EQ_OP_REG_SIZE VA_DEPTH                       /* Size of eq. op. reg. */
#define EQ_TRAIN_LEN 1500           /* Length of equalizer training sequence. */
#define EQ_TRN_2 1000                       /* Equalizer training: 2nd phase. */
#define EQ_STEP 1.0                                   /* Equalizer step size. */
#define EQ_TAP_PRINT (EQ_TRAIN_LEN+EQ_TRN_2) /* Time to print equalizer taps. */
#define VA_ON_TIME (EQ_TAP_PRINT+10)                   /* Turn on the VA now. */
#define TOT_TRN_LEN (VA_ON_TIME+VA_DELAY)        /* Total length of training. */
/******************************************************************************/
/*                SYSTEM DELAY PARAMETERS WRT PRESENT TIME.
*******************************************************************************/
#define DELTA (20)         /* Small adjustments for eq. to converge properly. */
#define EQ_OP_DELAY ((CHAN_SIZE>>SAMP_PER_SYM)+1+DELTA)        /* Eq. output. */
#define VA_OP_DELAY (EQ_OP_DELAY+VA_DEPTH-1)                    /* VA output. */
#define MAX_SYS_DELAY (VA_OP_DELAY+DF_SIZE)       /* Net. sys. delay with DF. */
/******************************************************************************/
/*                         ERROR BUFFER PARAMETERS.
*******************************************************************************/
#define ERR_BUF_SIZE (MAX_SYS_DELAY+100)             /* Size of error buffer. */
/******************************************************************************/
/*                   PARAMETERS RELATED TO THE VITERBI.
*******************************************************************************/
#define VA_DEPTH VA_DELAY                                 /* Depth of the VA. */
#define VA_OP_REG_SIZE (DF_SIZE+ONE)                /* Size of VA o/p buffer. */
#define F_FAC 1.0                                       /* Forgetting factor. */
#define THRESHOLD (1000000000.0)                        /* Squared threshold. */
/******************************************************************************/
/*                           STRUCTURE OF TRELLIS.
*******************************************************************************/
struct Trellis                                       /* Structure of trellis. */
{
 int St_Path[NUM_TX_STATES];                     /* Surviving path of states. */
 int Ip_Path[NUM_TX_STATES];                     /* Surviving path of inputs. */
 double VA_Wght[NUM_TX_STATES];                                    /* Metric. */
 struct Trellis *Prv_Depth;                     /* Address of previous depth. */
 struct Trellis *Nxt_Depth;                         /* Address of next depth. */
};
/******************************************************************************/
/*                             CONSTANTS.
*******************************************************************************/
#define BINSIZE 16         /* Size of array to store states in binary format. */
#define TWO 2
#define ONE 1
#define ZERO 0
#define TRUE 1
#define FALSE 0
#define LEFT 0
#define RIGHT 1
#define RE 0
#define IM 1
#define MINUSONE (-1)
#define ERROR 0.000001
#define FRAC_ZERO 0.0
#define MIN_MAG (-10.0)
#define MY_PI 3.14159265358979323846
#define PIBY2 1.57079632679489661923
#define PIBY4 0.785398163397448309615
#define TWICEPI 6.28318530717958647692
/******************************************************************************/
