/******************************************************************************/
/*   Copyright (C) 2001 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "pc4type.h"
#include "pc4ext.h"
/******************************************************************************/
/*                   Equalizer procedures in training mode.
*******************************************************************************/
int Equalizer_Procedures()
{
 Get_Equalizer_Op();
 if(Sym_Cnt <= EQ_TAP_PRINT)
 {
  Compute_Eq_Op_Error();
  Compute_Eq_Input_Energy();
  Update_Equalizer_Taps();
  if(Sym_Cnt == EQ_TAP_PRINT)
   Eq_Tap_Print_Procedures();
 }
 return(0);
}
/******************************************************************************/
/*                     Get the output of the equalizer.
*******************************************************************************/
int Get_Equalizer_Op()
{
 int cnt;
 Re_Eq_Op=Im_Eq_Op=0.0;
 for(cnt=ZERO;cnt<EQ_TAP_SIZE;cnt++)
 {
  Re_Eq_Op=Re_Eq_Op+Re_Eq_Reg[cnt]*Re_Eq_Tap[cnt];
  Re_Eq_Op=Re_Eq_Op-Im_Eq_Reg[cnt]*Im_Eq_Tap[cnt];
  Im_Eq_Op=Im_Eq_Op+Re_Eq_Reg[cnt]*Im_Eq_Tap[cnt];
  Im_Eq_Op=Im_Eq_Op+Im_Eq_Reg[cnt]*Re_Eq_Tap[cnt];
 }
 return(0);
}
/******************************************************************************/
/*              Compute the error at the equalizer output.
*******************************************************************************/
int Compute_Eq_Op_Error()
{
 int cnt,sym_num;
 double re_sym,im_sym,err_sq;
 sym_num=Eq_Err_Buf[LOOP_DELAY];
 re_sym=Re_Map[sym_num];
 im_sym=Im_Map[sym_num];
 Re_Eq_Err=re_sym-Re_Eq_Op;
 Im_Eq_Err=im_sym-Im_Eq_Op;
 if(Sym_Cnt > EQ_TRAIN_LEN)
  Store_Eq_Error();
 err_sq=Re_Eq_Err*Re_Eq_Err+Im_Eq_Err*Im_Eq_Err;
 if(Sym_Cnt < 10000)
  fprintf(fp3,"%lf\n",err_sq);
 return(0);
}
/******************************************************************************/
/*        Store the equalizer error for computing the autocorrelation.
*******************************************************************************/
int Store_Eq_Error()
{
 if(Eq_Err_Store_Inx < EQ_ERR_STORE_SIZE)
 {
  Re_Eq_Err_Store[Eq_Err_Store_Inx]=Re_Eq_Err;
  Im_Eq_Err_Store[Eq_Err_Store_Inx]=Im_Eq_Err;
  Eq_Err_Store_Inx++;
 }
 return(0);
}
/******************************************************************************/
/*                    Compute the equalizer input energy.
*******************************************************************************/
int Compute_Eq_Input_Energy()
{
 int cnt;
 Eq_Inp_Energy=0.0;
 for(cnt=ZERO;cnt<EQ_TAP_SIZE;cnt++)
 {
  Eq_Inp_Energy=Eq_Inp_Energy+Re_Eq_Reg[cnt]*Re_Eq_Reg[cnt];
  Eq_Inp_Energy=Eq_Inp_Energy+Im_Eq_Reg[cnt]*Im_Eq_Reg[cnt];
 }
 return(0);
}
/******************************************************************************/
/*                        Update the equalizer taps.
*******************************************************************************/
int Update_Equalizer_Taps()
{
 int cnt;
 double re_grad,im_grad,ss_fac;
 ss_fac=1.0;
 if(Sym_Cnt > EQ_TRAIN_LEN)
  ss_fac=0.1;
 for(cnt=ZERO;cnt<EQ_TAP_SIZE;cnt++)
 {
  re_grad=Re_Eq_Err*Re_Eq_Reg[cnt]+Im_Eq_Err*Im_Eq_Reg[cnt];
  im_grad=Im_Eq_Err*Re_Eq_Reg[cnt]-Re_Eq_Err*Im_Eq_Reg[cnt];
  Re_Eq_Tap[cnt]=Re_Eq_Tap[cnt]+ss_fac*EQ_STEP*re_grad/Eq_Inp_Energy;
  Im_Eq_Tap[cnt]=Im_Eq_Tap[cnt]+ss_fac*EQ_STEP*im_grad/Eq_Inp_Energy;
 }
 return(0);
}
/******************************************************************************/
/*                      Equalizer tap print procedures.
*******************************************************************************/
int Eq_Tap_Print_Procedures()
{
 Print_Eq_Taps();
 Get_Eq_Err_Autocorr();
 Print_Eq_Err_Autocorr();
 Get_Eq_Autocorr();
 Print_Eq_Autocorr();
 Levinson_Durbin();
 Eq_DD_Update_Instant=(Sym_Cnt+ONE)%LOOP_DELAY;
 printf("\t Equalizer taps update instant is %d.\n\n",Eq_DD_Update_Instant);
 return(0);
}
/******************************************************************************/
/*                        Print the equalizer taps.
*******************************************************************************/
int Print_Eq_Taps()
{
 int cnt;
 double energy,fac;
 energy=0.0;
 for(cnt=ZERO;cnt<EQ_TAP_SIZE;cnt++)
 {
  energy=energy+Re_Eq_Tap[cnt]*Re_Eq_Tap[cnt];
  energy=energy+Im_Eq_Tap[cnt]*Im_Eq_Tap[cnt];
 }
 fac=1.0/sqrt(energy);
 for(cnt=ZERO;cnt<EQ_TAP_SIZE;cnt++)
  fprintf(fp2,"%lf %lf\n",fac*Im_Eq_Tap[cnt],fac*Re_Eq_Tap[cnt]);
 return(0);
}
/******************************************************************************/
/*   Compute the autocorrelation of the error signal at the equalizer output.
*******************************************************************************/
int Get_Eq_Err_Autocorr()
{
 int size;
 double re[EQ_ERR_STORE_SIZE],im[EQ_ERR_STORE_SIZE];
 size=EQ_ERR_STORE_SIZE;
 Copy_Rev_Conjugate(Re_Eq_Err_Store,Im_Eq_Err_Store,re,im,EQ_ERR_STORE_SIZE);
 Convolve(Re_Eq_Err_Store,Im_Eq_Err_Store,re,im,Re_Err_AC,Im_Err_AC,size,size);
 Weight_AC();
 return(0);
}
/******************************************************************************/
/*                        Weight the autocorrelation.
*******************************************************************************/
int Weight_AC()
{
 int cnt;
 for(cnt=ZERO;cnt<ERR_AC_SIZE;cnt++)
 {
  if(cnt < EQ_ERR_STORE_SIZE)
  {
   Re_Err_AC[cnt]=Re_Err_AC[cnt]/(double)(cnt+ONE);
   Im_Err_AC[cnt]=Im_Err_AC[cnt]/(double)(cnt+ONE);
  }
  else
  {
   Re_Err_AC[cnt]=Re_Err_AC[cnt]/(double)(ERR_AC_SIZE-cnt);
   Im_Err_AC[cnt]=Im_Err_AC[cnt]/(double)(ERR_AC_SIZE-cnt);
  }
 }
 return(0);
}
/******************************************************************************/
/*                     Print the autocorrelation values.
*******************************************************************************/
int Print_Eq_Err_Autocorr()
{
 int cnt;
 for(cnt=ZERO;cnt<ERR_AC_SIZE;cnt++)
  fprintf(fp4,"%lf %lf\n",Im_Err_AC[cnt],Re_Err_AC[cnt]);
 return(0);
}
/******************************************************************************/
/*                     Get the equalizer autocorrelation.
*******************************************************************************/
int Get_Eq_Autocorr()
{
 int size;
 double re[EQ_TAP_SIZE],im[EQ_TAP_SIZE];
 size=EQ_TAP_SIZE;
 Copy_Rev_Conjugate(Re_Eq_Tap,Im_Eq_Tap,re,im,EQ_TAP_SIZE);
 Convolve(Re_Eq_Tap,Im_Eq_Tap,re,im,Re_Eq_AC,Im_Eq_AC,size,size);
 return(0);
}
/******************************************************************************/
/*                Print the autocorrelation of the equalizer.
*******************************************************************************/
int Print_Eq_Autocorr()
{
 int cnt;
 for(cnt=ZERO;cnt<EQ_AC_SIZE;cnt++)
  fprintf(fp5,"%lf %lf\n",Im_Eq_AC[cnt],Re_Eq_AC[cnt]);
 return(0);
}
/******************************************************************************/
