/******************************************************************************/
/*   Copyright (C) 2001 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "pc4type.h"
#include "pc4ext.h"
/******************************************************************************/
int Simulation()
{
 for(Sym_Cnt=ZERO;Sym_Cnt<MAX_SYM;Sym_Cnt++)
 {
  Get_Symbol();
  Get_Channel_Output();
  Add_Noise();
  Receive();
 }
 return(0);
}
/******************************************************************************/
/*                        Get the channel output.
*******************************************************************************/
int Get_Channel_Output()
{
 int cnt;
 Shift_Right(Re_Chan_Reg,Im_Chan_Reg,CHAN_SIZE);
 Re_Chan_Reg[ZERO]=Re_Sym;
 Im_Chan_Reg[ZERO]=Im_Sym;
 Re_Chan_Op=Im_Chan_Op=0.0;
 for(cnt=ZERO;cnt<CHAN_SIZE;cnt++)
 {
  Re_Chan_Op=Re_Chan_Op+Re_Chan_Reg[cnt]*Re_Chan_Tap[cnt];
  Re_Chan_Op=Re_Chan_Op-Im_Chan_Reg[cnt]*Im_Chan_Tap[cnt];
  Im_Chan_Op=Im_Chan_Op+Re_Chan_Reg[cnt]*Im_Chan_Tap[cnt];
  Im_Chan_Op=Im_Chan_Op+Im_Chan_Reg[cnt]*Re_Chan_Tap[cnt];
 }
 return(0);
}
/******************************************************************************/
/*                        Add white Gaussian noise.
*******************************************************************************/
int Add_Noise()
{
 double erand48();
 N_Val[URADIUS]=erand48(N1_Store);
 N_Val[UTHETA]=erand48(N2_Store);
 My_Gauss(VARIANCE,N_Val);
 Re_Noise=N_Val[GRAND_0];
 Im_Noise=N_Val[GRAND_1];
 Re_Chan_Op=Re_Chan_Op+Re_Noise;
 Im_Chan_Op=Im_Chan_Op+Im_Noise;
 return(0);
}
/******************************************************************************/
/*                         Receiver routines.
*******************************************************************************/
int Receive()
{
 Shift_Into_Equalizer();
 if(Sym_Cnt >= EQ_TRAIN_START)
 {
  Equalizer_Procedures();
  Shift_Into_Rx_Reg();
  Viterbi();
  Shift_Into_VA_Op_Buf();
//  if((Sym_Cnt>EQ_TAP_PRINT)&&((Sym_Cnt%LOOP_DELAY)==Eq_DD_Update_Instant))
  if(Sym_Cnt > EQ_TAP_PRINT)
   DD_Update_Eq_Taps();
  Update_Buffer_Procedures();
 }
 if(Sym_Cnt >= ERR_BUF_SHIFT_OUT_START)
  Update_Error();
 return(0);
}
/******************************************************************************/
/*               Shift the received sample into the equalizer.
*******************************************************************************/
int Shift_Into_Equalizer()
{
 Shift_Right(Re_Eq_Reg,Im_Eq_Reg,EQ_REG_SIZE);
 Re_Eq_Reg[ZERO]=Re_Chan_Op;
 Im_Eq_Reg[ZERO]=Im_Chan_Op;
 return(0);
}
/******************************************************************************/
/* Shift the equalizer output into the receiver register for processing by VA.
*******************************************************************************/
int Shift_Into_Rx_Reg()
{
 int cnt;
 for(cnt=RX_REG_SIZE-ONE;cnt>ZERO;cnt--)
 {
  Re_Rx_Reg[cnt]=Re_Rx_Reg[cnt-ONE];
  Im_Rx_Reg[cnt]=Im_Rx_Reg[cnt-ONE];
 }
 Re_Rx_Reg[ZERO]=Re_Eq_Op;
 Im_Rx_Reg[ZERO]=Im_Eq_Op;
 return(0);
}
/******************************************************************************/
/*                       Update the VA output buffer.
*******************************************************************************/
int Shift_Into_VA_Op_Buf()
{
 VA_Op_Buf[VA_Op_Buf_Inx]=VA_Op_Sym_Num;
 VA_Op_Buf_Inx++;
 if(VA_Op_Buf_Inx == ERR_BUF_SIZE)
  Error_Mes(6);
 return(0);
}
/******************************************************************************/
/*               Procedures for updating the error buffers.
*******************************************************************************/
int Update_Buffer_Procedures()
{
 Update_Eq_Err_Buf();
 if(Sym_Cnt >= VA_OP_BUF_SHIFT_OUT_START)
  Update_VA_Op_Buf();
 return(0);
}
/******************************************************************************/
/*                     Update the equalizer error buffer.
*******************************************************************************/
int Update_Eq_Err_Buf()
{
 int cnt;
 for(cnt=ZERO;cnt<=(Eq_Err_Buf_Inx-2);cnt++)
  Eq_Err_Buf[cnt]=Eq_Err_Buf[cnt+ONE];
 Eq_Err_Buf_Inx--;
 if(Eq_Err_Buf_Inx < ZERO)
  Error_Mes(7);
 return(0);
}
/******************************************************************************/
/*                       Update the VA output buffer.
*******************************************************************************/
int Update_VA_Op_Buf()
{
 int cnt;
 for(cnt=ZERO;cnt<=(VA_Op_Buf_Inx-2);cnt++)
  VA_Op_Buf[cnt]=VA_Op_Buf[cnt+ONE];
 VA_Op_Buf_Inx--;
 if(VA_Op_Buf_Inx < ZERO)
  Error_Mes(7);
 return(0);
}
/******************************************************************************/
/*                     Update the symbol error count.
*******************************************************************************/
int Update_Error()
{
 int cnt,inx;
 inx=LOOP_DELAY+ONE;
// if(Sym_Cnt > 9000)
//  printf("cnt:%d ebuf:%d va:%d\n",Sym_Cnt,Err_Buf[inx],VA_Ip_Sym_Num);
 if(Sym_Cnt >= EQ_TRAIN_LEN)
 {
  if(Err_Buf[inx] != VA_Ip_Sym_Num)
   Err_Cnt++;
  Valid_Sym++;
 }
 for(cnt=ZERO;cnt<=(Err_Buf_Inx-2);cnt++)
  Err_Buf[cnt]=Err_Buf[cnt+ONE];
 Err_Buf_Inx--;
 if(Err_Buf_Inx < ZERO)
  Error_Mes(7);
 return(0);
}
/******************************************************************************/
