/******************************************************************************/
/*   Copyright (C) 2001 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "pc4type.h"
#include "pc4ext.h"
/******************************************************************************/
/* Calculate the metric associated with each state for the current time
   instant.
*******************************************************************************/
int Calculate_Metric()
{
 if(Lst_Depth==NULL)
  Error_Mes(14);
 for(VA_P_St=ZERO;VA_P_St<NUM_TX_STATES;VA_P_St++)
 {
  for(VA_Inp=ZERO;VA_Inp<NUM_INPUTS;VA_Inp++)
  {
   VA_N_St=Tx_Trel_NS[VA_P_St][VA_Inp];
   Get_Curr_Metric();
   Get_Survivor();
  }
 }
 return(0);
}
/******************************************************************************/
/*        Get the surviving branch metric for the parallel transitions.
*******************************************************************************/
int Get_Curr_Metric()
{
 int op,cnt;
 double re_err,im_err,sum_re_err,sum_im_err,re_cf,im_cf;
 op=Tx_Trel_Op[VA_P_St][VA_Inp];
 sum_re_err=Re_Rx_Reg[ZERO]-Re_Map[op];
 sum_im_err=Im_Rx_Reg[ZERO]-Im_Map[op];
 Cur_Wght=sum_re_err*sum_re_err+sum_im_err*sum_im_err;
 return(0);
}
/******************************************************************************/
/*               Update the survivor to the next state.
*******************************************************************************/
int Get_Survivor()
{
 Get_New_Wght();
 Get_Existing_Wght();
 /*printf("PS:%d inp:%d NS:%d ",VA_P_St,VA_Inp,VA_N_St);
 printf("oldwght:%lf newwght:%lf\n",Old_Wght,New_Wght);*/
 if(New_Wght < Old_Wght)
  Survivor_Info();
 return(0);
}
/******************************************************************************/
/*                    Get the new metric at next state.
*******************************************************************************/
int Get_New_Wght()
{
 struct Trellis *prev;
 prev=Lst_Depth->Prv_Depth;
 if(prev!=NULL)
  Cur_Wght=F_FAC*(prev->VA_Wght[VA_P_St])+Cur_Wght;
 New_Wght=Cur_Wght;
 /*printf("mod pst:%d inp:%d nst:%d ",VA_P_St,VA_Inp,VA_N_St);
 printf("cur wght:%lf ",Cur_Wght);*/
 return(0);
}
/******************************************************************************/
/*                    Get existing metric at next state.
*******************************************************************************/
int Get_Existing_Wght()
{
 Old_Wght=Lst_Depth->VA_Wght[VA_N_St];
 return(0);
}
/******************************************************************************/
/*                    Get survivor for the next state.
*******************************************************************************/
int Survivor_Info()
{
 Lst_Depth->VA_Wght[VA_N_St]=Cur_Wght;
 Lst_Depth->St_Path[VA_N_St]=VA_P_St;
 Lst_Depth->Ip_Path[VA_N_St]=VA_Inp;
 /*printf("PS:%d inp:%d NS:%d ",VA_P_St,VA_Inp,VA_N_St);
 printf("oldwght:%lf newwght:%lf\n",Old_Wght,New_Wght);*/
 return(0);
}
/******************************************************************************/
/*                           Print the metrics.
*******************************************************************************/
int Print_Metrics()
{
 int cnt;
 for(cnt=ZERO;cnt<NUM_TX_STATES;cnt++)
 {
  printf("Sym_Cnt:%d st:%d ",Sym_Cnt,cnt);
  printf("rewght:%lf\n",Lst_Depth->VA_Wght[cnt]);
 }
 return(0);
}
/******************************************************************************/
/*                Get the state having the minimum weight.
*******************************************************************************/
int Get_Min_Wght()
{
 int cnt;
 double min;
 min=THRESHOLD;
 Min_Wght_St=ZERO;
 for(cnt=ZERO;cnt<NUM_TX_STATES;cnt++)
 {
  if(Lst_Depth->VA_Wght[cnt] < min)
  {
   min=Lst_Depth->VA_Wght[cnt];
   Min_Wght_St=cnt;
  }
 }
 return(0);
}
/******************************************************************************/
/*                Go backwards and get the last symbol in time.
*******************************************************************************/
int Get_Last_Symbol()
{
 int tstate,tip;
 struct Trellis *curr;
 curr=Lst_Depth;
 tstate=Min_Wght_St;
 /*printf("Backtracking:\n");*/
 while(curr!=NULL)
 {
  tip=curr->Ip_Path[tstate];
  /*printf("currst:%d ip:%d ",tstate,tip);*/
  tstate=curr->St_Path[tstate];
  /*printf("Prevst:%d\n",tstate);*/
  curr=curr->Prv_Depth;
 }
 VA_Ip_Sym_Num=tip;                                     /* Ip. symbol number. */
 VA_Op_Sym_Num=Tx_Trel_Op[tstate][tip];                 /* Op. symbol number. */
 /*printf("Released symbol is %d.\n",VA_Op_Sym_Num);*/
 return(0);
}
/******************************************************************************/
/*                       Reinitialize Linked List.
*******************************************************************************/
int Linked_List_Reinit()
{
 struct Trellis *tnext;
 tnext=Fst_Depth->Nxt_Depth;
 tnext->Prv_Depth=NULL;
 Lst_Depth->Nxt_Depth=Fst_Depth;
 Fst_Depth->Prv_Depth=Lst_Depth;
 Fst_Depth->Nxt_Depth=NULL;
 Lst_Depth=Fst_Depth;
 Fst_Depth=tnext;
 return(0);
}
/******************************************************************************/
/*               Reinitialize metrics at the last depth.
*******************************************************************************/
int Metric_Reinit()
{
 int cnt;
 for(cnt=ZERO;cnt<NUM_TX_STATES;cnt++)
  Lst_Depth->VA_Wght[cnt]= THRESHOLD;
 return(0);
}
/******************************************************************************/
