/******************************************************************************/
/*   Copyright (C) 2001 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
/*                       FILE POINTER VARIABLES.
*******************************************************************************/
FILE *fp1,*fp2,*fp3,*fp4,*fp5,*fp6;                         /* File pointers. */
/******************************************************************************/
/*                          COUNTER VARIABLES.
*******************************************************************************/
long Sym_Cnt;                                                /* Symbol count. */
long Valid_Sym;                                        /* Valid symbol count. */
/******************************************************************************/
/*                      BIT AND SYMBOL VARIABLES.
*******************************************************************************/
int Inp_Sym_Num;                                /* Sym. no. i/p to conv. enc. */
int Op_Sym_Num;                               /* Sym. no. o/p from conv. enc. */
int P_State;                                     /* Present state of encoder. */
double Re_Sym;                                 /* Re. part of current symbol. */
double Im_Sym;                                 /* Im. part of current symbol. */
double Re_Rx_Reg[RX_REG_SIZE];                         /* Stored Re. symbols. */
double Im_Rx_Reg[RX_REG_SIZE];                         /* Stored Im. symbols. */
double Re_Map[CONSTELL_SIZE];                         /* Re. part of symbols. */
double Im_Map[CONSTELL_SIZE];                         /* Im. part of symbols. */
/******************************************************************************/
/*                    VARIABLES RELATED TO ERROR BUFFER.
*******************************************************************************/
int Err_Cnt;                                      /* No. of symbols in error. */
int Err_Buf_Inx;                                    /* Index to error buffer. */
int Err_Buf[ERR_BUF_SIZE];                                   /* Error buffer. */
/******************************************************************************/
/*                           CHANNEL VARIABLES.
*******************************************************************************/
double Chan_Energy;                                        /* Channel energy. */
double Re_Chan_Op;                                     /* Re. channel output. */
double Im_Chan_Op;                                     /* Im. channel output. */
double Re_Chan_Tap[CHAN_SIZE];                           /* Re. channel taps. */
double Im_Chan_Tap[CHAN_SIZE];                           /* Im. channel taps. */
double Re_Chan_Reg[CHAN_SIZE];                            /* Re. channel reg. */
double Im_Chan_Reg[CHAN_SIZE];                            /* Im. channel reg. */
/******************************************************************************/
/*                       NOISE GENERATOR VARIABLES.
*******************************************************************************/
unsigned short N1_Store[PN_SIZE];                       /* Stores PNS status. */
unsigned short N2_Store[PN_SIZE];                       /* Stores PNS status. */
double N_Val[NVALSIZE];                          /* Contains U & G rand. var. */
double Re_Noise;                               /* Re. part of coloured noise. */
double Im_Noise;                               /* Im. part of coloured noise. */
double Ip_Coeff;                                        /* IIR coeff. of i/p. */
double Coeff_Array[PRED_MEM_SIZE];                      /* IIR coeff. of o/p. */
double Re_N_Reg[PRED_MEM_SIZE];                        /* Re. col. noise reg. */
double Im_N_Reg[PRED_MEM_SIZE];                        /* Im. col. noise reg. */
/******************************************************************************/
/*                          EQUALIZER VARIABLES.
*******************************************************************************/
int Eq_DD_Update_Instant;            /* Update instant of the eq. in DD mode. */
int Eq_Err_Buf_Inx;                       /* Index to equalizer error buffer. */
int Eq_Err_Buf[ERR_BUF_SIZE];                      /* Equalizer error buffer. */
double Eq_Inp_Energy;                           /* Energy of equalizer input. */
double Re_Eq_Err;                              /* Re. error at equalizer o/p. */
double Im_Eq_Err;                              /* Im. error at equalizer o/p. */
double Re_Eq_Op;                                        /* Re. equalizer o/p. */
double Im_Eq_Op;                                        /* Re. equalizer o/p. */
double Re_Eq_Tap[EQ_TAP_SIZE];                         /* Re. equalizer taps. */
double Im_Eq_Tap[EQ_TAP_SIZE];                         /* Im. equalizer taps. */
double Re_Eq_Reg[EQ_REG_SIZE];                          /* Re. equalizer reg. */
double Im_Eq_Reg[EQ_REG_SIZE];                          /* Im. equalizer reg. */
/******************************************************************************/
/*            VARIABLES RELATED TO THE LEVINSON DURBIN ALGORITHM.
*******************************************************************************/
int Eq_Err_Store_Inx;                               /* Eq. error store index. */
double Re_Eq_Err_Store[EQ_ERR_STORE_SIZE];           /* Stores re. eq. error. */
double Im_Eq_Err_Store[EQ_ERR_STORE_SIZE];           /* Stores im. eq. error. */
double Pred_Var;                                /* Prediction error variance. */
double Re_Pred_Coeff[NUM_PRED][PRED_SIZE];                /* Re. pred. coeff. */
double Im_Pred_Coeff[NUM_PRED][PRED_SIZE];                /* Im. pred. coeff. */
double Re_Err_AC[ERR_AC_SIZE];                   /* Re. autocorr. of eq. err. */
double Im_Err_AC[ERR_AC_SIZE];                   /* Im. autocorr. of eq. err. */
double Re_Eq_AC[EQ_AC_SIZE];                          /* Re. autocorr. of eq. */
double Im_Eq_AC[EQ_AC_SIZE];                          /* Im. autocorr. of eq. */
double Re_AC[PRED_SIZE];                                     /* Re. autocorr. */
double Im_AC[PRED_SIZE];                                     /* Im. autocorr. */
/******************************************************************************/
/*                    VARIABLES RELATED TO THE TRELLIS.
*******************************************************************************/
int Tx_Trel_NS[NUM_TX_STATES][NUM_INPUTS];                     /* Tx:Nxt. st. */
int Tx_Trel_Op[NUM_TX_STATES][NUM_INPUTS];                          /* Tx:Op. */
int Rx_Trel_NS[MOD_NUM_RX_STATES][NUM_INPUTS];                 /* Rx:Nxt. st. */
int Rx_Trel_Op[MOD_NUM_RX_STATES][NUM_INPUTS][PRED_SIZE];           /* Rx:Op. */
int Mod_State_Map[NUM_RX_STATES];          /* Mapping to modified rx. states. */
int Iv_Mod_State_Map[NUM_RX_STATES];                      /* Inverse mapping. */
int Ck_Map[NUM_RX_STATES];                                      /* Map check. */
int Mod_State;                                             /* Modified state. */
/******************************************************************************/
/*                    VARIABLES RELATED TO THE VITERBI.
*******************************************************************************/
int Depth;                              /* Gives the current depth of the VA. */
int VA_Inp;                                      /* Current input to a state. */
int VA_P_St;                                                /* Current state. */
int VA_N_St;                                                   /* Next state. */
int Min_Wght_St;                          /* State having the maximum weight. */
int VA_Ip_Sym_Num;                              /* Ip. symbol released by VA. */
int VA_Op_Sym_Num;                              /* Op. symbol released by VA. */
int VA_Op_Buf_Inx;                              /* Index to VA output buffer. */
int VA_Op_Buf[ERR_BUF_SIZE];            /* Buffer to store VA output symbols. */
double Cur_Wght;                                            /* Cur. distance. */
double New_Wght;                                             /* New net wght. */
double Old_Wght;                                             /* Old net wght. */
struct Trellis *Fst_Depth,*Lst_Depth;                           /* Templates. */
/******************************************************************************/
