/******************************************************************************/
/*   Copyright (C) 2001 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
/******************************************************************************/
/*                    SIMULATION RUN LENGTH PARAMETER.
*******************************************************************************/
#define VA_DELAY 40                                       /* Delay in the VA. */
#define MAX_SYM (10000000+EQ_TRAIN_LEN)          /* Length of simulation run. */
//#define MAX_SYM (10000000+EQ_TAP_PRINT)        /* Length of simulation run. */
/******************************************************************************/
/*                         CONSTELLATION PARAMETERS.
*******************************************************************************/
#define CONSTELL_SIZE 4                             /* Size of constellation. */
/******************************************************************************/
/*                         ERROR BUFFER PARAMETERS.
*******************************************************************************/
#define ERR_BUF_SIZE 256                             /* Size of error buffer. */
/******************************************************************************/
/*                          TRELLIS PARAMETERS.
*******************************************************************************/
#define NUM_TX_STATES 4              /* No. of states in transmitter trellis. */
#define NUM_PRED_STATES 16   /* No. of states corresponding to the predictor. */
#define NUM_RX_STATES 64               /* No. of states in receiver trellis. */
#define NUM_INPUTS 2                                        /* No. of inputs. */
#define MOD_NUM_INPUTS 8                          /* Modified no. of inputs. */
#define MOD_NUM_RX_STATES 16                   /* Modified no. of rx. states. */
/******************************************************************************/
/*                          CHANNEL PARAMETERS.
*******************************************************************************/
#define CHAN_SIZE 5                                   /* No. of channel taps. */
/******************************************************************************/
/*                  GAUSSIAN NOISE GENERATOR PARAMETERS.
*******************************************************************************/
#define WHITE 1                    /* 1:Add white noise;0:add coloured noise. */
#define VARIANCE 00.140                                    /* Noise variance. */
#define PN_SIZE 3                 /* Size of array which stores the PN value. */
#define NVALSIZE 4
#define URADIUS 0
#define UTHETA 1
#define GRAND_0 2
#define GRAND_1 3
/******************************************************************************/
/*                       RECEIVER REGISTER PARAMETERS.
*******************************************************************************/
#define RX_REG_SIZE LOOP_DELAY                  /* Size of receiver register. */
/******************************************************************************/
/*                          EQUALIZER PARAMETERS.
*******************************************************************************/
#define DD_MODE 1             /* 1:Decision directed mode; 0:correct symbols. */
#define EQ_TAP_SIZE 31                              /* No. of equalizer taps. */
#define EQ_REG_SIZE (EQ_TAP_SIZE+LOOP_DELAY)        /* Size of equalizer reg. */
#define EQ_TRAIN_LEN 1000           /* Length of equalizer training sequence. */
#define EQ_STEP 1.0                                   /* Equalizer step size. */
#define EQ_TAP_PRINT (EQ_TRAIN_LEN+1500)     /* Time to print equalizer taps. */
#define DELTA_DELAY 2                           /* Loop delay above VA delay. */
#define LOOP_DELAY (VA_DELAY+DELTA_DELAY)    /* Delay of the loop in DD mode. */
/******************************************************************************/
/*                 TIME TO START TRAINING THE EQUALIZER.
*******************************************************************************/
#define EQ_TRAIN_START (((EQ_TAP_SIZE+CHAN_SIZE)>> 1)+9+LOOP_DELAY)
/******************************************************************************/
/*          TIME TO START SHIFTING OUT CONTENTS OF THE ERROR BUFFER.
*******************************************************************************/
#define ERR_BUF_SHIFT_OUT_START (EQ_TRAIN_START+VA_DELAY)
/******************************************************************************/
/*        TIME TO START SHIFTING OUT CONTENTS OF THE VA OUTPUT BUFFER.
*******************************************************************************/
#define VA_OP_BUF_SHIFT_OUT_START (EQ_TRAIN_START+DELTA_DELAY)
/******************************************************************************/
/*      PARAMETERS RELATED TO THE PREDICTOR REQUIRED TO WHITEN THE NOISE.
*******************************************************************************/
#define PRED_MEM_SIZE 2                      /* Size of the predictor memory. */
/******************************************************************************/
/*           PARAMETERS RELATED TO THE LEVINSON DURBIN ALGORITHM.
*******************************************************************************/
#define EQ_AC 0              /* 1:Use eq. autocorr; 0:use eq. error autocorr. */
#define NUM_PRED (PRED_MEM_SIZE+1)                   /* Number of predictors. */
#define PRED_SIZE (PRED_MEM_SIZE+1)                /* Size of the predictors. */
#define EQ_ERR_STORE_SIZE 1000     /* Size of array to store equalizer error. */
#define ERR_AC_SIZE ((EQ_ERR_STORE_SIZE << 1)-1)     /* Autocorr. of eq. err. */
#define EQ_AC_SIZE ((EQ_TAP_SIZE << 1)-1)                 /* Autocorr. of eq. */
/******************************************************************************/
/*                   PARAMETERS RELATED TO THE VITERBI.
*******************************************************************************/
#define VA_DEPTH VA_DELAY                                 /* Depth of the VA. */
#define F_FAC 1.0                                       /* Forgetting factor. */
#define THRESHOLD (1000000000.0)                        /* Squared threshold. */
/******************************************************************************/
/*                           STRUCTURE OF TRELLIS.
*******************************************************************************/
struct Trellis                                       /* Structure of trellis. */
{
 int St_Path[MOD_NUM_RX_STATES];                 /* Surviving path of states. */
 int Ip_Path[MOD_NUM_RX_STATES];                 /* Surviving path of inputs. */
 double VA_Wght[MOD_NUM_RX_STATES];                                /* Metric. */
 struct Trellis *Prv_Depth;                     /* Address of previous depth. */
 struct Trellis *Nxt_Depth;                         /* Address of next depth. */
};
/******************************************************************************/
/*                             CONSTANTS.
*******************************************************************************/
#define BINSIZE 16         /* Size of array to store states in binary format. */
#define TWO 2
#define ONE 1
#define ZERO 0
#define TRUE 1
#define FALSE 0
#define LEFT 0
#define RIGHT 1
#define RE 0
#define IM 1
#define MINUSONE (-1)
#define ERROR 0.000001
#define FRAC_ZERO 0.0
#define MIN_MAG (-10.0)
#define MY_PI 3.14159265358979323846
#define PIBY2 1.57079632679489661923
#define PIBY4 0.785398163397448309615
#define TWICEPI 6.28318530717958647692
/******************************************************************************/
