/******************************************************************************/
/*   Copyright (C) 2001 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "pc6type.h"
#include "pc6ext.h"
/******************************************************************************/
/*           Get the prediction coefficients from the LD algorithm.
*******************************************************************************/
int Levinson_Durbin()
{
 int order_cnt;
 Init_LD();
 for(order_cnt=ONE;order_cnt<=PRED_MEM_SIZE;order_cnt++)
 {
  Get_Reflection_Coeff(order_cnt);
  Get_New_Coeff(order_cnt);
  Get_New_Pred_Var(order_cnt);
  Print_Pred_Info(order_cnt);
 }
 return(0);
}
/******************************************************************************/
/*                       Initialize the LD algorithm.
*******************************************************************************/
int Init_LD()
{
 int order,inx;
 Pred_Var=Re_AC[ZERO];                           /* Zero lag autocorrelation. */
 for(order=ZERO;order<=PRED_MEM_SIZE;order++)
 {
  Re_Pred_Coeff[order][ZERO]=1.0;
  Im_Pred_Coeff[order][ZERO]=0.0;
  for(inx=ONE;inx<=PRED_MEM_SIZE;inx++)
   Re_Pred_Coeff[order][inx]=Im_Pred_Coeff[order][inx]=0.0;
 }
 return(0);
}
/******************************************************************************/
/*            Get the reflection coefficient for the current order.
*******************************************************************************/
int Get_Reflection_Coeff(order_cnt)
int order_cnt;
{
 int cnt,ac_inx;
 double re_numer,im_numer;
 re_numer=im_numer=0.0;
 for(cnt=ZERO;cnt<order_cnt;cnt++)
 {
  ac_inx=order_cnt-cnt;
  re_numer=re_numer+Re_Pred_Coeff[order_cnt-ONE][cnt]*Re_AC[ac_inx];
  re_numer=re_numer-Im_Pred_Coeff[order_cnt-ONE][cnt]*Im_AC[ac_inx];
  im_numer=im_numer+Re_Pred_Coeff[order_cnt-ONE][cnt]*Im_AC[ac_inx];
  im_numer=im_numer+Im_Pred_Coeff[order_cnt-ONE][cnt]*Re_AC[ac_inx];
 }
 Re_Pred_Coeff[order_cnt][order_cnt]= -re_numer/Pred_Var;
 Im_Pred_Coeff[order_cnt][order_cnt]= -im_numer/Pred_Var;
 printf("\t Reflection coeff. for order %d is ",order_cnt);
 printf("Re:%14.12lf ",Re_Pred_Coeff[order_cnt][order_cnt]);
 printf("Im:%14.12lf.\n\n",Im_Pred_Coeff[order_cnt][order_cnt]);
 return(0);
}
/******************************************************************************/
/*             Get the predictor coefficients for the next order.
*******************************************************************************/
int Get_New_Coeff(order_cnt)
int order_cnt;
{
 int cnt;
 double re_rc,im_rc,re_x,im_x,re_cf,im_cf;
 re_rc=Re_Pred_Coeff[order_cnt][order_cnt];
 im_rc=Im_Pred_Coeff[order_cnt][order_cnt];
 for(cnt=ONE;cnt<order_cnt;cnt++)
 {
  re_cf=Re_Pred_Coeff[order_cnt-ONE][order_cnt-cnt];
  im_cf=Im_Pred_Coeff[order_cnt-ONE][order_cnt-cnt];
  re_x=re_rc*re_cf+im_rc*im_cf;
  im_x=im_rc*re_cf-re_rc*im_cf;
  Re_Pred_Coeff[order_cnt][cnt]=Re_Pred_Coeff[order_cnt-ONE][cnt]+re_x;
  Im_Pred_Coeff[order_cnt][cnt]=Im_Pred_Coeff[order_cnt-ONE][cnt]+im_x;
 }
 return(0);
}
/******************************************************************************/
/*                  Get the new prediction error variance.
*******************************************************************************/
int Get_New_Pred_Var(order_cnt)
int order_cnt;
{
 int cnt;
 double re_var,im_var,re_cf,im_cf,re_ac,im_ac;
 re_var=im_var=0.0;
 for(cnt=ZERO;cnt<=order_cnt;cnt++)
 {
  re_cf=Re_Pred_Coeff[order_cnt][cnt];
  im_cf=Im_Pred_Coeff[order_cnt][cnt];
  re_ac=  Re_AC[cnt];
  im_ac= -Im_AC[cnt];                                        /* Negative lag. */
  re_var=re_var+re_cf*re_ac-im_cf*im_ac;
  im_var=im_var+re_cf*im_ac+im_cf*re_ac;
 }
 Pred_Var=re_var;
 return(0);
}
/******************************************************************************/
/*                     Print the prediction information.
*******************************************************************************/
int Print_Pred_Info(order_cnt)
int order_cnt;
{
 int cnt;
 printf("\t Prediction coefficients for order %d:\n",order_cnt);
 for(cnt=ZERO;cnt<=order_cnt;cnt++)
 {
  printf("\t\t Inx:%d Re_Coeff:%14.12lf  ",cnt,Re_Pred_Coeff[order_cnt][cnt]);
  printf("Im_Coeff:%14.12lf\n",Im_Pred_Coeff[order_cnt][cnt]);
 }
 printf("\n");
 printf("\t Pred. variance of order %d is :%14.12lf.\n",order_cnt,Pred_Var);
 printf("\n");
 return(0);
}
/******************************************************************************/
