/******************************************************************************/
/*   Copyright (C) 2001 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
/******************************************************************************/
/*                    SIMULATION RUN LENGTH PARAMETER.
*******************************************************************************/
#define VA_DELAY 40                                      /* Delay in the VA. */
#define MAX_SYM (10000000+TOT_TRN_LEN+1)         /* Length of simulation run. */
/******************************************************************************/
/*                         CONSTELLATION PARAMETERS.
*******************************************************************************/
#define CONSTELL_SIZE 4                             /* Size of constellation. */
#define SAMP_PER_SYM 2                                 /* Samples per symbol. */
/******************************************************************************/
/*                          TRELLIS PARAMETERS.
*******************************************************************************/
#define NUM_TX_STATES 4              /* No. of states in transmitter trellis. */
#define NUM_PRED_STATES 64   /* No. of states corresponding to the predictor. */
#define NUM_RX_STATES 256               /* No. of states in receiver trellis. */
#define NUM_INPUTS 2                                        /* No. of inputs. */
#define MOD_NUM_INPUTS 16                          /* Modified no. of inputs. */
#define MOD_NUM_RX_STATES 32                   /* Modified no. of rx. states. */
/******************************************************************************/
/*                          CHANNEL PARAMETERS.
*******************************************************************************/
#define CHAN_SIZE 12                                  /* No. of channel taps. */
/******************************************************************************/
/*                  GAUSSIAN NOISE GENERATOR PARAMETERS.
*******************************************************************************/
#define WHITE 1                    /* 1:Add white noise;0:add coloured noise. */
#define VARIANCE 00.1000                                   /* Noise variance. */
#define PN_SIZE 3                 /* Size of array which stores the PN value. */
#define NVALSIZE 4
#define URADIUS 0
#define UTHETA 1
#define GRAND_0 2
#define GRAND_1 3
/******************************************************************************/
/*                          EQUALIZER PARAMETERS.
*******************************************************************************/
#define DD_MODE 1                  /* 1:Decision directed; 0:correct symbols. */
#define EQ_TAP_SIZE 51            /* No. of equalizer taps; feedforward part. */
#define EXTRA_EQ_REG_LEN (SAMP_PER_SYM*(VA_DEPTH-1))         /* Extra memory. */
#define EQ_REG_SIZE (EQ_TAP_SIZE+EXTRA_EQ_REG_LEN)  /* Size of equalizer reg. */
#define EQ_OP_REG_SIZE VA_DEPTH                       /* Size of eq. op. reg. */
#define EQ_TRAIN_LEN 1500           /* Length of equalizer training sequence. */
#define EQ_TRN_2 1000                     /* Equalizer training: 2nd phase. */
#define EQ_STEP 1.0                                   /* Equalizer step size. */
#define EQ_TAP_PRINT (EQ_TRAIN_LEN+EQ_TRN_2) /* Time to print equalizer taps. */
#define VA_ON_TIME (EQ_TAP_PRINT+10)                   /* Turn on the VA now. */
#define TOT_TRN_LEN (VA_ON_TIME+VA_DELAY)        /* Total length of training. */
/******************************************************************************/
/*                SYSTEM DELAY PARAMETERS WRT PRESENT TIME.
*******************************************************************************/
#define DELTA (11)         /* Small adjustments for eq. to converge properly. */
#define EQ_OP_DELAY ((CHAN_SIZE>>SAMP_PER_SYM)+1+DELTA)        /* Eq. output. */
#define VA_OP_DELAY (EQ_OP_DELAY+VA_DEPTH-1)                    /* VA output. */
#define MAX_SYS_DELAY VA_OP_DELAY                       /* Net. system delay. */
/******************************************************************************/
/*                         ERROR BUFFER PARAMETERS.
*******************************************************************************/
#define ERR_BUF_SIZE (MAX_SYS_DELAY+100)             /* Size of error buffer. */
/******************************************************************************/
/*      PARAMETERS RELATED TO THE PREDICTOR REQUIRED TO WHITEN THE NOISE.
*******************************************************************************/
#define PRED_MEM_SIZE 3                      /* Size of the predictor memory. */
/******************************************************************************/
/*           PARAMETERS RELATED TO THE LEVINSON DURBIN ALGORITHM.
*******************************************************************************/
#define EQ_AC 0                      /* 1:eq. autocorr; 0:eq. error autocorr. */
#define NUM_PRED (PRED_MEM_SIZE+1)                   /* Number of predictors. */
#define PRED_SIZE (PRED_MEM_SIZE+1)                /* Size of the predictors. */
#define EQ_ERR_STORE_SIZE (EQ_TRN_2)       /* Size of array to store eq. err. */
#define ERR_AC_SIZE ((EQ_ERR_STORE_SIZE << 1)-1)     /* Autocorr. of eq. err. */
#define EQ_AC_SIZE ((EQ_TAP_SIZE << 1)-1)                 /* Autocorr. of eq. */
/******************************************************************************/
/*                   PARAMETERS RELATED TO THE VITERBI.
*******************************************************************************/
#define VA_DEPTH VA_DELAY                                 /* Depth of the VA. */
#define F_FAC 1.0                                       /* Forgetting factor. */
#define THRESHOLD (1000000000.0)                        /* Squared threshold. */
/******************************************************************************/
/*                           STRUCTURE OF TRELLIS.
*******************************************************************************/
struct Trellis                                       /* Structure of trellis. */
{
 int St_Path[MOD_NUM_RX_STATES];                 /* Surviving path of states. */
 int Ip_Path[MOD_NUM_RX_STATES];                 /* Surviving path of inputs. */
 double VA_Wght[MOD_NUM_RX_STATES];                                /* Metric. */
 struct Trellis *Prv_Depth;                     /* Address of previous depth. */
 struct Trellis *Nxt_Depth;                         /* Address of next depth. */
};
/******************************************************************************/
/*                             CONSTANTS.
*******************************************************************************/
#define BINSIZE 16         /* Size of array to store states in binary format. */
#define TWO 2
#define ONE 1
#define ZERO 0
#define TRUE 1
#define FALSE 0
#define LEFT 0
#define RIGHT 1
#define RE 0
#define IM 1
#define MINUSONE (-1)
#define ERROR 0.000001
#define FRAC_ZERO 0.0
#define MIN_MAG (-10.0)
#define MY_PI 3.14159265358979323846
#define PIBY2 1.57079632679489661923
#define PIBY4 0.785398163397448309615
#define TWICEPI 6.28318530717958647692
/******************************************************************************/
