/******************************************************************************/
/*   Copyright (C) 2001 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "pc7type.h"
#include "pc7ext.h"
/******************************************************************************/
/*                 Get the transmitter and receiver trellis.
*******************************************************************************/
int Get_Trellis()
{
 Get_Tx_Trellis();
 Get_Rx_Trellis();
 Ck_Mod_State();
 printf("\n");
}
/******************************************************************************/
/*                      Get the transmitter trellis.
*******************************************************************************/
int Get_Tx_Trellis()
{
 int p_state,p_input;
 for(p_state=ZERO;p_state<NUM_TX_STATES;p_state++)
 {
  for(p_input=ZERO;p_input<NUM_INPUTS;p_input++)
  {
   Tx_Trel_Op[p_state][p_input]=Get_Tx_Trel_Op(p_state,p_input);
   Tx_Trel_NS[p_state][p_input]=(2*p_state+p_input)%NUM_TX_STATES;
   printf("\t Tx. trellis Pre. state:%3d Input:%3d ",p_state,p_input);
   printf("Nxt. state:%3d ",Tx_Trel_NS[p_state][p_input]);
   printf("Op:%3d.\n",Tx_Trel_Op[p_state][p_input]);
  }
 }
 printf("\n");
 return(0);
}
/******************************************************************************/
/*                   Get the output. LSB maps to real part.
*******************************************************************************/
int Get_Tx_Trel_Op(p_state,p_input)
int p_state,p_input;
{
 int bit0,bit1,op0,op1,dec_op;
 bit0=p_state&1;
 bit1=(p_state/2)&1;
 op0=p_input^bit1;
 op1=p_input^bit0^bit1;
 dec_op=op0+2*op1;
 return(dec_op);
}
/******************************************************************************/
/*                      Get the trellis at the receiver.
*******************************************************************************/
int Get_Rx_Trellis()
{
 int p_state,p_input,ns[NUM_RX_STATES][NUM_INPUTS];
 int op[NUM_RX_STATES][NUM_INPUTS][PRED_SIZE];
 for(p_state=ZERO;p_state<NUM_TX_STATES;p_state++)
 {
  for(p_input=ZERO;p_input<MOD_NUM_INPUTS;p_input++)
   Process_Rx_Trel(p_state,p_input,ns,op);
 }
 Modify_Rx_Trellis(ns,op);
 return(0);
}
/******************************************************************************/
/*      Process the receiver trellis for a given encoder state and input.
*******************************************************************************/
int Process_Rx_Trel(p_state,p_input,ns,op)
int p_state,p_input,ns[NUM_RX_STATES][NUM_INPUTS];
int op[NUM_RX_STATES][NUM_INPUTS][PRED_SIZE];
{
 int enc_state,pred_state,net_p_state,pred_st_array[PRED_MEM_SIZE];
 int cnt,inp,ip_st_array[PRED_SIZE];
 Decimal_To_M_ary(p_input,ip_st_array,PRED_SIZE,2); /* Index 0 is oldest bit. */
 enc_state=p_state;
 for(cnt=ZERO;cnt<PRED_MEM_SIZE;cnt++)
 {
  inp=ip_st_array[cnt];
  pred_st_array[PRED_MEM_SIZE-ONE-cnt]=Tx_Trel_Op[enc_state][inp];
  enc_state=Tx_Trel_NS[enc_state][inp];
 }
 net_p_state=Get_Net_P_State(enc_state,pred_st_array);
 Get_Rx_Trel_Op(net_p_state,enc_state,ip_st_array,pred_st_array,op);
 Get_Rx_Trel_NS(net_p_state,enc_state,ip_st_array,pred_st_array,ns);
 return(0);
}
/******************************************************************************/
/*                     Get the resultant present state.
*******************************************************************************/
int Get_Net_P_State(enc_state,pred_st_array)
int enc_state,pred_st_array[];
{
 int net_p_state,pred_state;
 pred_state=M_ary_To_Decimal(pred_st_array,PRED_MEM_SIZE,CONSTELL_SIZE);
 net_p_state=enc_state*NUM_PRED_STATES+pred_state;
 return(net_p_state);
}
/******************************************************************************/
/* Get the output for the receiver trellis, given the input and the
   current state.
*******************************************************************************/
int Get_Rx_Trel_Op(net_p_state,enc_state,ip_st_array,pred_st_array,op)
int net_p_state,enc_state,ip_st_array[],pred_st_array[];
int op[NUM_RX_STATES][NUM_INPUTS][PRED_SIZE];
{
 int cnt,sym,inp;
 inp=ip_st_array[PRED_SIZE-ONE];                          /* Most recent bit. */
 op[net_p_state][inp][ZERO]=Tx_Trel_Op[enc_state][inp];
 for(cnt=ZERO;cnt<PRED_MEM_SIZE;cnt++)
  op[net_p_state][inp][cnt+ONE]=pred_st_array[cnt];
 return(0);
}
/******************************************************************************/
/* Get the next state for the receiver trellis, given the input and the
   net current state.
*******************************************************************************/
int Get_Rx_Trel_NS(net_p_state,enc_state,ip_st_array,pred_st_array,ns)
int net_p_state,enc_state,ip_st_array[],pred_st_array[];
int ns[NUM_RX_STATES][NUM_INPUTS];
{
 int inp,cnt,pred_state,enc_op,enc_ns,net_n_state;
 inp=ip_st_array[PRED_SIZE-ONE];                          /* Most recent bit. */
 enc_op=Tx_Trel_Op[enc_state][inp];
 enc_ns=Tx_Trel_NS[enc_state][inp];
 for(cnt=PRED_MEM_SIZE-ONE;cnt>ZERO;cnt--)
  pred_st_array[cnt]=pred_st_array[cnt-ONE];
 pred_st_array[ZERO]=enc_op;
 pred_state=M_ary_To_Decimal(pred_st_array,PRED_MEM_SIZE,CONSTELL_SIZE);
 net_n_state=enc_ns*NUM_PRED_STATES+pred_state;
 ns[net_p_state][inp]=net_n_state;
 Get_Mod_State_Mapping(net_n_state);
 return(0);
}
/******************************************************************************/
/*                     Get the modified state mapping.
*******************************************************************************/
int Get_Mod_State_Mapping(net_n_state)
int net_n_state;
{
 if(Ck_Map[net_n_state] == FALSE)
 {
  Mod_State_Map[net_n_state]=Mod_State;
  Iv_Mod_State_Map[Mod_State]=net_n_state;
  Ck_Map[net_n_state]=TRUE;
  Mod_State++;
 }
 return(0);
}
/******************************************************************************/
/*                       Modify the receiver trellis.
*******************************************************************************/
int Modify_Rx_Trellis(ns,op)
int ns[NUM_RX_STATES][NUM_INPUTS];
int op[NUM_RX_STATES][NUM_INPUTS][PRED_SIZE];
{
 int state,ip,act_state,n_state,act_n_state,inx;
 for(state=ZERO;state<MOD_NUM_RX_STATES;state++)
 {
  act_state=Iv_Mod_State_Map[state];
  for(ip=ZERO;ip<NUM_INPUTS;ip++)
  {
   act_n_state=ns[act_state][ip];
   n_state=Mod_State_Map[act_n_state];
   Rx_Trel_NS[state][ip]=n_state;
   for(inx=ZERO;inx<PRED_SIZE;inx++)
    Rx_Trel_Op[state][ip][inx]=op[act_state][ip][inx];
   Print_Rx_Trellis(state,ip);
  }
 }
 return(0);
}
/******************************************************************************/
/*                        Print the receiver trelis.
*******************************************************************************/
int Print_Rx_Trellis(p_state,p_input)
int p_state,p_input;
{
 int inx;
 printf("\t Rx. trellis Pre. state:%3d Input:%3d ",p_state,p_input);
 printf("Nxt. state:%3d\n",Rx_Trel_NS[p_state][p_input]);
 printf("\t Ops:\n");
 for(inx=ZERO;inx<PRED_SIZE;inx++)
  printf("\t inx:%d  Sym:%d\n",inx,Rx_Trel_Op[p_state][p_input][inx]);
 return(0);
}
/******************************************************************************/
/*                  Check the value of the modified state.
*******************************************************************************/
int Ck_Mod_State()
{
 if(Mod_State != MOD_NUM_RX_STATES)
  Error_Mes(12);
 return(0);
}
/******************************************************************************/
