/******************************************************************************/
/*   Copyright (C) 1993 K. Vasudevan. Indian Institute of Technology Madras.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/*  This program computes the FFT of the given data file.                     */
/*  The data file must have two columns. The 1st column denotes time          */
/*  (for real valued data) or the imaginary part (for complex data). The      */
/*  second column denotes the real part of the data (for both real or         */
/*  complex data).                                                            */
/******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
/******************************************************************************/
#define DATASIZE 32768
#define FILESIZE 128
#define ONE 1
#define ZERO 0
#define TRUE 1
#define FALSE 0
#define REAL 1
#define BINSIZE 16
#define REVERSED 1
#define UNREVERSED 0
#define PI 3.1415926
#define DELTA 0.00001
/******************************************************************************/
#define HAMMING 2
/******************************************************************************/
int datapts,numpts,status,expo,twopowexp,winopt,grdstart;
char inpfile[FILESIZE],magfile[FILESIZE],angfile[FILESIZE];
char grdfile[FILESIZE];
double SAMPFREQ,LIMMAGRESP,LIMGRDRESP,reinp[DATASIZE],iminp[DATASIZE];
double gain,prevang;
FILE *fp1,*fp2,*fp3,*fp4;
/******************************************************************************/
main()
{
 getinputs();                          /* Get name & status of the data file. */
 initinput();                                 /* Initialize all data to zero. */
 readfile();                                               /* Read data file. */
 preprocdata();                                       /* Preprocess the data. */
 getexpo();                           /* Get log2[N],rounded to next integer. */
 bitreverse();                            /* Place data in bit reversed form. */
 fft();                                                       /* Compute FFT. */
 resultprint();                             /* Print result into output file. */
 myclose();                                                   /* Close files. */
}
/******************************************************************************/
/*                          Get all the inputs.
*******************************************************************************/
int getinputs()
{
 printf("Enter input file:");
 scanf("%s",inpfile);
 strcpy(magfile,inpfile);
 strcpy(angfile,inpfile);
 strcpy(grdfile,inpfile);
 strcat(inpfile,".dat");
 strcat(magfile,"mag.dat");
 strcat(angfile,"ang.dat");
 strcat(grdfile,"grd.dat");
 myopen();                             /* Open files for reading and writing. */
 printf("Real data [1] or Complex data [0]?:");
 scanf("%d",&status);
 printf("Enter the sampling frequency in kHz:");
 scanf("%lf",&SAMPFREQ);
 printf("Enter the Magnitude response limits in kHz (<0.5*SAMPFREQ):");
 scanf("%lf",&LIMMAGRESP);
 printf("Enter the group delay response limits in kHz (<0.5*SAMPFREQ):");
 scanf("%lf",&LIMGRDRESP);
 givwinopt();
 printf("Enter gain:");
 scanf("%lf",&gain);
 return(0);
}
/******************************************************************************/
/*                    Open the input file for reading.
*******************************************************************************/
int myopen()
{
 fp1=fopen(inpfile,"r");
 if(fp1==NULL)
  errormes(1);
 fp2=fopen(magfile,"w");
 fp3=fopen(angfile,"w");
 fp4=fopen(grdfile,"w");
 return(0);
}
/******************************************************************************/
/*           Print the window options available to window the data.
*******************************************************************************/
int givwinopt()
{
 printf("These are the Windows available to window the data.\n");
 printf("  RECTANGULAR WINDOW -- [1].\n");
 printf("  HAMMING WINDOW     -- [2].\n");
 printf("Select Window Option:");
 scanf("%d",&winopt);
 return(0);
}
/******************************************************************************/
int initinput()
{
 int cnt;
 for(cnt=ZERO;cnt<DATASIZE;cnt++)
 {
  reinp[cnt]=0.0;
  iminp[cnt]=0.0;
 }
 return(0);
}
/******************************************************************************/
/*                       Read data from the input file.
*******************************************************************************/
int readfile()
{
 numpts=0;
 while(fscanf(fp1,"%lf %lf\n",&iminp[numpts],&reinp[numpts])!=EOF)
 {
  /*printf("Real data:%f Imaginary data:%f\n",reinp[i],iminp[i]);*/
  numpts++;
  if(numpts==DATASIZE)
   errormes(2);
 }
 if(numpts==ZERO)
  errormes(3);
 datapts=numpts;                         /* Store the actual no. of data pts. */
 printf("No. of points given:%d\n",numpts);
 printf("Change to:");
 scanf("%d",&numpts);
 return(0);
}
/******************************************************************************/
/*                                 Preprocess the data.
*******************************************************************************/
int preprocdata()
{
 int cnt;
 if(status==REAL)                                               /* Real data. */
 {
  for(cnt=ZERO;cnt<DATASIZE;cnt++)     /* Initialize irrelevant data to zero. */
   iminp[cnt]=0.0;
 }
 if(winopt==HAMMING)
  hammproc();
 return(0);
}
/******************************************************************************/
/*                   Window the non-zero part of the data.
*******************************************************************************/
hammproc()
{
 int cnt;
 double n,delay,theta,w;
 if((datapts%2)==1)
  delay=(double)(datapts/2);
 else
  delay=(double)(datapts/2)-0.5;
 n=0.0;
 printf("No. of data pts.:%d",datapts);
 for(cnt=ZERO;cnt<datapts;cnt++)
 {
  theta=(n-delay)*PI/delay;
  w=0.54+0.46*cos(theta);
  reinp[cnt]=reinp[cnt]*w;
  iminp[cnt]=iminp[cnt]*w;
  n=n+1.0;
 }
 return(0);
}
/******************************************************************************/
int getexpo()
{
 int loop;
 expo=0;
 twopowexp=1;
 loop=TRUE;
 printf("       This program computes a maximum of %d-point FFT.\n",DATASIZE);
 printf("       Number of points given: %d\n",numpts);
 while(loop==TRUE)
 {
  if(twopowexp>=numpts)
   loop=FALSE;
  else
  {
   twopowexp=twopowexp*2;
   expo++;
  }
 }
 printf("       This is to be converted to %d-point FFT\n",twopowexp);
 printf("       Exponent:%d 2**expo:%d\n",expo,twopowexp);
 if(twopowexp>DATASIZE)
  errormes(2);
 return(0);
}
/******************************************************************************/
/*                             Do bit reversal.
*******************************************************************************/
int bitreverse()
{
 int bin[BINSIZE],cnt,flag[DATASIZE],cnt1;
 double tmp;
 initflag(flag,twopowexp);              /* Initialize 'flag[]' to UNREVERSED. */
 for(cnt=ONE;cnt<twopowexp;cnt++)
 {
  if(flag[cnt]==UNREVERSED)
  {
   binary(cnt,bin,BINSIZE);
   reverse(bin,expo);
   cnt1=decimal(bin,expo);
   tmp=reinp[cnt];
   reinp[cnt]=reinp[cnt1];
   reinp[cnt1]=tmp;
   tmp=iminp[cnt];
   iminp[cnt]=iminp[cnt1];
   iminp[cnt1]=tmp;
   flag[cnt]=flag[cnt1]=REVERSED;
  }
 }
 return(0);
}
/******************************************************************************/
int initflag(flag,size)
int flag[],size;
{
 int cnt;
 for(cnt=ZERO;cnt<size;cnt++)
  flag[cnt]=UNREVERSED;
 return(0);
}
/******************************************************************************/
/*           Convert decimal to binary.Store in array'bin[size]'.
*******************************************************************************/
int binary(num,bin,size)
int num,bin[],size;
{
 int i,quotient;
 for(i=ZERO;i<size;i++)
  bin[i]=ZERO;
 quotient=num;
 i=ZERO;
 while(quotient>ZERO)
 {
  if(i==size)
   errormes(4);
  bin[i]=quotient%2;
  i++;
  quotient=quotient/2;
 }
 return(0);
}
/******************************************************************************/
/*       Re-arrange in bit reversed order.Store result in array 'bin[]'.
*******************************************************************************/
int reverse(bin,expo)
int bin[],expo;
{
 int rem,tmp,half,lcnt,ucnt;
 half=expo/2-1;
 ucnt=expo-1;
 for(lcnt=ZERO;lcnt<=half;lcnt++)
 {
  tmp=bin[lcnt];
  bin[lcnt]=bin[ucnt];
  bin[ucnt]=tmp;
  ucnt--;
 }
 return(0);
}
/******************************************************************************/
/*               Convert binary stored in 'bin[]' to decimal.
*******************************************************************************/
int decimal(bin,expo)
int bin[],expo;
{
 int sum,cnt,wt;
 wt=1;
 sum=0;
 for(cnt=ZERO;cnt<expo;cnt++)
 {
  sum=sum+wt*bin[cnt];
  wt=wt*2;
 }
 return(sum);
}
/******************************************************************************/
/*                     Do Decimation in Time FFT.
*******************************************************************************/
int fft()
{
 int itop,ibot,size,sizem2,s,j,stage,ispace,iwidth;
 double theta,rewt,imwt,retmp,imtmp;
 ispace=ONE;
 size=twopowexp;
 sizem2=size-2;
 for(stage=ONE;stage<=expo;stage++)            /* Compute for log2[N] stages. */
 {
  iwidth=ispace;    /* 'iwidth':Sep. of pts. that participate in a butterfly. */
  ispace=2*ispace;  /* 'ispace':Sep. of top(or bottom) pts. in consecutive -- */
  s=size/ispace;             /* --butterflies that use same weighting factor. */
  for(j=ZERO;j<iwidth;j++)
  {
   theta=PI*(double)j/(double)iwidth;
   rewt=cos(theta);                         /* Real part of weighting factor. */
   imwt=0.0-sin(theta);                     /* Ima. part of weighting factor. */
   for(itop=j;itop<=sizem2;itop=itop+ispace)        /* Butterfly computation. */
   {
    ibot=itop+iwidth;
    retmp=reinp[ibot]*rewt-iminp[ibot]*imwt;                       /* Re(bc). */
    imtmp=iminp[ibot]*rewt+reinp[ibot]*imwt;                       /* Im(bc). */
    reinp[ibot]=reinp[itop]-retmp;                     /* Re(b)=Re(a)-Re(bc). */
    iminp[ibot]=iminp[itop]-imtmp;                     /* Im(b)=Im(a)-Im(bc). */
    reinp[itop]=reinp[itop]+retmp;                     /* Re(a)=Re(a)+Re(bc). */
    iminp[itop]=iminp[itop]+imtmp;                     /* Im(a)=Im(a)+Im(bc). */
   }
  }
 }
 return(0);
}
/******************************************************************************/
/*                         Print the results.
*******************************************************************************/
int resultprint()
{
 prevang=0.0;
 grdstart=TRUE;                              /* Start computing the gradient. */
 if(LIMMAGRESP>(0.5*SAMPFREQ))
  errormes(5);
 if(LIMGRDRESP>(0.5*SAMPFREQ))
  errormes(5);
 negfreqplot();
 posfreqplot();
 return(0);
}
/******************************************************************************/
/*                  Get plots for negative frequencies.
*******************************************************************************/
int negfreqplot()
{
 int cnt,limit;
 double mag,ang,freq,dtheta,slope,tlim;
 double getang();
 tlim=LIMMAGRESP*(double)(twopowexp)/SAMPFREQ;
 limit=(int)tlim;
 for(cnt=(twopowexp-limit);cnt<twopowexp;cnt++)
 {
  mag=reinp[cnt]*reinp[cnt]+iminp[cnt]*iminp[cnt];
  ang=getang(cnt);
  if((ang>0.0)&&(prevang<0.0))
   dtheta=prevang-ang+2.0*PI;
  else
   dtheta=prevang-ang;
  prevang=ang;
  slope=dtheta*(double)(twopowexp)/(2.0*PI);
  freq=(double)cnt*SAMPFREQ/(double)twopowexp-SAMPFREQ;
  fprintf(fp2,"%lf %lf\n",freq,mag);
  fprintf(fp3,"%lf %lf\n",freq,ang);
  if((freq> -LIMGRDRESP) && (grdstart == FALSE))   /* Do not print 1st slope. */
   fprintf(fp4,"%lf %lf\n",freq,slope);
  grdstart=FALSE;                                   /* 1st gradient computed. */
 }
 return(0);
}
/******************************************************************************/
/*                  Get plots for positive frequencies.
*******************************************************************************/
int posfreqplot()
{
 int cnt,limit;
 double mag,ang,freq,dtheta,slope,tlim;
 double getang();
 tlim=LIMMAGRESP*(double)(twopowexp)/SAMPFREQ;
 limit=(int)tlim;
 for(cnt=ZERO;cnt<=limit;cnt++)
 {
  mag=reinp[cnt]*reinp[cnt]+iminp[cnt]*iminp[cnt];
  ang=getang(cnt);
  if((ang>0.0)&&(prevang<0.0))
   dtheta=prevang-ang+2.0*PI;
  else
   dtheta=prevang-ang;
  prevang=ang;
  slope=dtheta*(double)(twopowexp)/(2.0*PI);
  freq=(double)cnt*SAMPFREQ/(double)twopowexp;
  fprintf(fp2,"%lf %lf\n",freq,mag);
  fprintf(fp3,"%lf %lf\n",freq,ang);
  if(freq<LIMGRDRESP)
   fprintf(fp4,"%lf %lf\n",freq,slope);
 }
 return(0);
}
/******************************************************************************/
/*                        Get the principal angle.
*******************************************************************************/
double getang(cnt)
int cnt;
{
 double ang;
 if((reinp[cnt]<DELTA)&&(reinp[cnt]> -DELTA)&&(iminp[cnt]>=0.0))
  ang=0.5*PI;
 else if((reinp[cnt]<DELTA)&&(reinp[cnt]> -DELTA)&&(iminp[cnt]<0.0))
  ang= -0.5*PI;
 else
 {
  ang=atan(iminp[cnt]/reinp[cnt]);
  if((reinp[cnt]<0.0)&&(iminp[cnt]>0.0))
   ang=PI+ang;
  else if((reinp[cnt]<0.0)&&(iminp[cnt]<0.0))
   ang= -PI+ang;
 }
 return(ang);
}
/******************************************************************************/
int myclose()
{
 fclose(fp1);
 fclose(fp2);
 fclose(fp3);
 fclose(fp4);
 return(0);
}
/******************************************************************************/
int errormes(num)
int num;
{
 if(num==1)
  fprintf(stdout,"\t FILE %s DOES NOT EXIST! EXIT!\n",inpfile);
 else if(num==2)
  fprintf(stdout,"\t ARRAY SIZE INSUFFICIENT! EXIT!\n");
 else if(num==3)
  fprintf(stdout,"\t EMPTY FILE! EXIT!\n");
 else if(num==4)
  fprintf(stdout,"\t DECIMAL TO BINARY ARRAY SIZE INSUFFICIENT! EXIT!\n");
 else if(num==5)
  fprintf(stdout,"\t INCORRECT RESPONSE LIMITS! EXIT!\n");
 exit(num);
 return(0);
}
/******************************************************************************/
