/******************************************************************************/
/*   Copyright (C) 2001 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "pc8type.h"
#include "pc8ext.h"
/******************************************************************************/
/*                   Equalizer procedures in training mode.
*******************************************************************************/
int Equalizer_Trn_Procedures()
{
 Compute_Eq_Op_Error();
 Compute_Eq_Input_Energy();
 Update_FF_Eq_Taps();
 Update_DF_Eq_Taps();
 if(Sym_Cnt == EQ_TAP_PRINT)
  Eq_Tap_Print_Procedures();
 return(0);
}
/******************************************************************************/
/*              Compute the error at the equalizer output.
*******************************************************************************/
int Compute_Eq_Op_Error()
{
 int cnt,sym_num,inx;
 double re_sym,im_sym,err_sq;
 inx=Eq_Err_Buf_Inx-EQ_OP_DELAY-ONE;
 if(inx < ZERO)
  Error_Mes(9);
 sym_num=Eq_Err_Buf[inx];
 re_sym=Re_Map[sym_num];
 im_sym=Im_Map[sym_num];
 Re_Eq_Err=re_sym-Re_Eq_Op;
 Im_Eq_Err=im_sym-Im_Eq_Op;
 if(Sym_Cnt > EQ_TRAIN_LEN)
  Store_Eq_Error();
 err_sq=Re_Eq_Err*Re_Eq_Err+Im_Eq_Err*Im_Eq_Err;
 fprintf(fp3,"%lf\n",err_sq);
 return(0);
}
/******************************************************************************/
/*        Store the equalizer error for computing the autocorrelation.
*******************************************************************************/
int Store_Eq_Error()
{
 if(Eq_Err_Store_Inx < EQ_ERR_STORE_SIZE)
 {
  Re_Eq_Err_Store[Eq_Err_Store_Inx]=Re_Eq_Err;
  Im_Eq_Err_Store[Eq_Err_Store_Inx]=Im_Eq_Err;
  Eq_Err_Store_Inx++;
 }
 return(0);
}
/******************************************************************************/
/*                    Compute the equalizer input energy.
*******************************************************************************/
int Compute_Eq_Input_Energy()
{
 int cnt;
 Eq_Inp_Energy=0.0;
 for(cnt=ZERO;cnt<EQ_TAP_SIZE;cnt++)
 {
  Eq_Inp_Energy=Eq_Inp_Energy+Re_Eq_Reg[cnt]*Re_Eq_Reg[cnt];
  Eq_Inp_Energy=Eq_Inp_Energy+Im_Eq_Reg[cnt]*Im_Eq_Reg[cnt];
 }
 return(0);
}
/******************************************************************************/
/*                  Update the feedforward equalizer taps.
*******************************************************************************/
int Update_FF_Eq_Taps()
{
 int cnt;
 double re_grad,im_grad,ss_fac;
 ss_fac=0.5;
 if(Sym_Cnt > EQ_TRAIN_LEN)
  ss_fac=0.10;
 for(cnt=ZERO;cnt<EQ_TAP_SIZE;cnt++)
 {
  re_grad=Re_Eq_Err*Re_Eq_Reg[cnt]+Im_Eq_Err*Im_Eq_Reg[cnt];
  im_grad=Im_Eq_Err*Re_Eq_Reg[cnt]-Re_Eq_Err*Im_Eq_Reg[cnt];
  Re_Eq_Tap[cnt]=Re_Eq_Tap[cnt]+ss_fac*EQ_STEP*re_grad/Eq_Inp_Energy;
  Im_Eq_Tap[cnt]=Im_Eq_Tap[cnt]+ss_fac*EQ_STEP*im_grad/Eq_Inp_Energy;
 }
 return(0);
}
/******************************************************************************/
/*               Update the decision feedback equalizer taps.
*******************************************************************************/
int Update_DF_Eq_Taps()
{
 int cnt,inx,sym_num;
 double re_grad,im_grad,ss_fac,re_sym,im_sym;
 ss_fac=0.5;
 if(Sym_Cnt > EQ_TRAIN_LEN)
  ss_fac=0.10;
 for(cnt=ZERO;cnt<DF_SIZE;cnt++)
 {
  inx=Eq_Err_Buf_Inx-EQ_OP_DELAY-ONE-ONE-cnt;
  if(inx < ZERO)
   Error_Mes(9);
  sym_num=Eq_Err_Buf[inx];
  re_sym=Re_Map[sym_num];
  im_sym=Im_Map[sym_num];
  re_grad=Re_Eq_Err*re_sym+Im_Eq_Err*im_sym;
  im_grad=Im_Eq_Err*re_sym-Re_Eq_Err*im_sym;
  Re_DF_Tap[cnt]=Re_DF_Tap[cnt]+ss_fac*EQ_STEP*re_grad/(double)(2*DF_SIZE);
  Im_DF_Tap[cnt]=Im_DF_Tap[cnt]+ss_fac*EQ_STEP*im_grad/(double)(2*DF_SIZE);
 }
 return(0);
}
/******************************************************************************/
/*                      Equalizer tap print procedures.
*******************************************************************************/
int Eq_Tap_Print_Procedures()
{
 Ck_Eq_Err_Inx();
 Print_Eq_Taps();
 Print_DF_Taps();
 Get_Eq_Err_Autocorr();
 Print_Eq_Err_Autocorr();
 Get_Eq_Autocorr();
 Print_Eq_Autocorr();
 Get_Final_Autocorr();
 Levinson_Durbin();
 return(0);
}
/******************************************************************************/
/*                     Check the equalizer error index.
*******************************************************************************/
int Ck_Eq_Err_Inx()
{
 if(Eq_Err_Store_Inx != EQ_ERR_STORE_SIZE)
  Error_Mes(9);
 return(0);
}
/******************************************************************************/
/*                        Print the equalizer taps.
*******************************************************************************/
int Print_Eq_Taps()
{
 int cnt;
 double energy,fac;
 energy=0.0;
 for(cnt=ZERO;cnt<EQ_TAP_SIZE;cnt++)
 {
  energy=energy+Re_Eq_Tap[cnt]*Re_Eq_Tap[cnt];
  energy=energy+Im_Eq_Tap[cnt]*Im_Eq_Tap[cnt];
 }
 fac=1.0/sqrt(energy);
 for(cnt=ZERO;cnt<EQ_TAP_SIZE;cnt++)
  fprintf(fp2,"%lf %lf\n",fac*Im_Eq_Tap[cnt],fac*Re_Eq_Tap[cnt]);
 return(0);
}
/******************************************************************************/
/*                   Print the decision feedback taps.
*******************************************************************************/
int Print_DF_Taps()
{
 int cnt;
 double energy,fac;
 energy=0.0;
 for(cnt=ZERO;cnt<DF_SIZE;cnt++)
 {
  energy=energy+Re_DF_Tap[cnt]*Re_DF_Tap[cnt];
  energy=energy+Im_DF_Tap[cnt]*Im_DF_Tap[cnt];
 }
 fac=1.0/sqrt(energy);
 for(cnt=ZERO;cnt<DF_SIZE;cnt++)
  fprintf(fp7,"%lf %lf\n",fac*Im_DF_Tap[cnt],fac*Re_DF_Tap[cnt]);
 return(0);
}
/******************************************************************************/
/*   Compute the autocorrelation of the error signal at the equalizer output.
*******************************************************************************/
int Get_Eq_Err_Autocorr()
{
 int size;
 double re[EQ_ERR_STORE_SIZE],im[EQ_ERR_STORE_SIZE];
 size=EQ_ERR_STORE_SIZE;
 Copy_Rev_Conjugate(Re_Eq_Err_Store,Im_Eq_Err_Store,re,im,size);
 Convolve(Re_Eq_Err_Store,Im_Eq_Err_Store,re,im,Re_Err_AC,Im_Err_AC,size,size);
 Weight_AC();
 return(0);
}
/******************************************************************************/
/*                        Weight the autocorrelation.
*******************************************************************************/
int Weight_AC()
{
 int cnt;
 for(cnt=ZERO;cnt<ERR_AC_SIZE;cnt++)
 {
  if(cnt < EQ_ERR_STORE_SIZE)
  {
   Re_Err_AC[cnt]=Re_Err_AC[cnt]/(double)(cnt+ONE);
   Im_Err_AC[cnt]=Im_Err_AC[cnt]/(double)(cnt+ONE);
  }
  else
  {
   Re_Err_AC[cnt]=Re_Err_AC[cnt]/(double)(ERR_AC_SIZE-cnt);
   Im_Err_AC[cnt]=Im_Err_AC[cnt]/(double)(ERR_AC_SIZE-cnt);
  }
 }
 return(0);
}
/******************************************************************************/
/*                     Print the autocorrelation values.
*******************************************************************************/
int Print_Eq_Err_Autocorr()
{
 int cnt;
 printf("\t AC peak:Im:%lf ",Im_Err_AC[EQ_ERR_STORE_SIZE-ONE]);
 printf(" Re:%lf\n\n",Re_Err_AC[EQ_ERR_STORE_SIZE-ONE]);
 for(cnt=ZERO;cnt<ERR_AC_SIZE;cnt++)
  fprintf(fp4,"%lf %lf\n",Im_Err_AC[cnt],Re_Err_AC[cnt]);
 return(0);
}
/******************************************************************************/
/*                     Get the equalizer autocorrelation.
*******************************************************************************/
int Get_Eq_Autocorr()
{
 int size;
 double re[EQ_TAP_SIZE],im[EQ_TAP_SIZE];
 size=EQ_TAP_SIZE;
 Copy_Rev_Conjugate(Re_Eq_Tap,Im_Eq_Tap,re,im,EQ_TAP_SIZE);
 Convolve(Re_Eq_Tap,Im_Eq_Tap,re,im,Re_Eq_AC,Im_Eq_AC,size,size);
 return(0);
}
/******************************************************************************/
/*                Print the autocorrelation of the equalizer.
*******************************************************************************/
int Print_Eq_Autocorr()
{
 int cnt;
 for(cnt=ZERO;cnt<EQ_AC_SIZE;cnt++)
  fprintf(fp5,"%lf %lf\n",Im_Eq_AC[cnt],Re_Eq_AC[cnt]);
 return(0);
}
/******************************************************************************/
/*                     Get the final autocorrelation.
*******************************************************************************/
int Get_Final_Autocorr()
{
 if(EQ_AC)
  Copy_Eq_Autocorr();
 else
  Copy_Err_Autocorr();
 return(0);
}
/******************************************************************************/
/*     Copy the equalizer autocorrelation into the autocorrelation array.
*******************************************************************************/
int Copy_Eq_Autocorr()
{
 int cnt;
 for(cnt=ZERO;cnt<(PRED_MEM_SIZE+ONE);cnt++)
 {
  Re_AC[cnt]=Re_Eq_AC[EQ_TAP_SIZE-ONE+cnt];
  Im_AC[cnt]=Im_Eq_AC[EQ_TAP_SIZE-ONE+cnt];
 }
 return(0);
}
/******************************************************************************/
/*  Copy the equalizer error autocorrelation into the autocorrelation array.
*******************************************************************************/
int Copy_Err_Autocorr()
{
 int cnt;
 for(cnt=ZERO;cnt<(PRED_MEM_SIZE+ONE);cnt++)
 {
  Re_AC[cnt]=Re_Err_AC[EQ_ERR_STORE_SIZE-ONE+cnt];
  Im_AC[cnt]=Im_Err_AC[EQ_ERR_STORE_SIZE-ONE+cnt];
 }
 return(0);
}
/******************************************************************************/
