/******************************************************************************/
/*   Copyright (C) 2001 K. Vasudevan. Indian Institute of Technology Kanpur.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
/******************************************************************************/
/*                    SIMULATION RUN LENGTH PARAMETER.
*******************************************************************************/
#define TRN_LENGTH 100                                 /* Length of training. */
#define VA_DELAY 40                                       /* Delay in the VA. */
#define MAX_SYM (10000000+VA_DELAY-1+TRN_LENGTH)     /* Length of simulation. */
/******************************************************************************/
/*                         CONSTELLATION PARAMETERS.
*******************************************************************************/
#define CONSTELL_SIZE 4                             /* Size of constellation. */
/******************************************************************************/
/*                         ERROR BUFFER PARAMETERS.
*******************************************************************************/
#define ERR_BUF_SIZE 64                              /* Size of error buffer. */
/******************************************************************************/
/*                          TRELLIS PARAMETERS.
*******************************************************************************/
#define NUM_TX_STATES 4              /* No. of states in transmitter trellis. */
#define NUM_PRED_STATES 64   /* No. of states corresponding to the predictor. */
#define NUM_RX_STATES 256               /* No. of states in receiver trellis. */
#define NUM_INPUTS 2                                        /* No. of inputs. */
#define MOD_NUM_INPUTS 16                          /* Modified no. of inputs. */
#define MOD_NUM_RX_STATES 32                   /* Modified no. of rx. states. */
/******************************************************************************/
/*                  GAUSSIAN NOISE GENERATOR PARAMETERS.
*******************************************************************************/
#define WHITE 0                    /* 1:Add white noise;0:add coloured noise. */
#define VARIANCE 1.30                                      /* Noise variance. */
#define PN_SIZE 3                 /* Size of array which stores the PN value. */
#define NVALSIZE 4
#define URADIUS 0
#define UTHETA 1
#define GRAND_0 2
#define GRAND_1 3
#define AR_MEM_SIZE 1                /* Size of the memory of the AR process. */
/******************************************************************************/
/*      PARAMETERS RELATED TO THE PREDICTOR REQUIRED TO WHITEN THE NOISE.
*******************************************************************************/
#define ERR_AC_SIZE (2*TRN_LENGTH-ONE)    /* Length of autocorr. of training. */
#define PRED_MEM_SIZE 3                      /* Size of the predictor memory. */
#define PRED_SIZE (PRED_MEM_SIZE+1)                 /* Size of the predictor. */
/******************************************************************************/
/*           PARAMETERS RELATED TO THE LEVINSON DURBIN ALGORITHM.
*******************************************************************************/
#define NUM_PRED (PRED_MEM_SIZE+1)                   /* Number of predictors. */
/******************************************************************************/
/*                   PARAMETERS RELATED TO THE VITERBI.
*******************************************************************************/
#define VA_DEPTH VA_DELAY                                 /* Depth of the VA. */
#define F_FAC 1.0                                       /* Forgetting factor. */
#define THRESHOLD (1000000000.0)                        /* Squared threshold. */
/******************************************************************************/
/*                           STRUCTURE OF TRELLIS.
*******************************************************************************/
struct Trellis                                       /* Structure of trellis. */
{
 int St_Path[MOD_NUM_RX_STATES];                 /* Surviving path of states. */
 int Ip_Path[MOD_NUM_RX_STATES];                 /* Surviving path of inputs. */
 double VA_Wght[MOD_NUM_RX_STATES];                                /* Metric. */
 struct Trellis *Prv_Depth;                     /* Address of previous depth. */
 struct Trellis *Nxt_Depth;                         /* Address of next depth. */
};
/******************************************************************************/
/*                             CONSTANTS.
*******************************************************************************/
#define BINSIZE 16         /* Size of array to store states in binary format. */
#define TWO 2
#define ONE 1
#define ZERO 0
#define TRUE 1
#define FALSE 0
#define LEFT 0
#define RIGHT 1
#define RE 0
#define IM 1
#define MINUSONE (-1)
#define ERROR 0.000001
#define FRAC_ZERO 0.0
#define MIN_MAG (-10.0)
#define MY_PI 3.14159265358979323846
#define PIBY2 1.57079632679489661923
#define PIBY4 0.785398163397448309615
#define TWICEPI 6.28318530717958647692
/******************************************************************************/
